@@ -3,7 +3,7 @@
 """
 Driver script for performing an global optimization over the parameter space.
 """
-# Copyright (C) 2011, 2012 University of Zurich. All rights reserved.
+# Copyright (C) 2011, 2012  University of Zurich. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +19,6 @@ Driver script for performing an global optimization over the parameter space.
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #
-__version__ = '$Revision$'
 __author__ = 'Riccardo Murri <riccardo.murri@uzh.ch>, Benjamin Jonen <benjamin.jonen@bf.uzh.ch>'
 # summary of user-visible changes
 __changelog__ = """
@@ -27,6 +26,14 @@ __changelog__ = """
 """
 __docformat__ = 'reStructuredText'
 
+# std module imports
+import numpy as np
+import os
+import re
+import shutil
+import sys
+import time
+import datetime
 
 # Calling syntax one4all:
 # -b ~/workspace/fpProj/model/base/ -x ~/workspace/fpProj/model/bin/forwardPremiumOut -e ~/workspace/fpProj/empirical/ -C 16 -NP 8 -xVars 'EA sigmaA' -xVarsDom '0.8 0.94 0.004 0.006' -sv info -t one4all --itermax 1 --countryList 'AU DE' -J 1000 -X i686 -r localhost --norm "fpSqr2" --initialPop "" -N
@@ -40,23 +47,14 @@ if __name__ == '__main__':
         path2Pymods = os.path.join(os.path.dirname(__file__), '../')
         if not sys.path.count(path2Pymods):
             sys.path.append(path2Pymods)
-        from pymods.support.support import rmFilesAndFolders
         curPath = os.getcwd()
         filesAndFolder = os.listdir(curPath)
         if 'gParaSearch.csv' in filesAndFolder: # if another paraSearch was run in here before, clean up.
-            rmFilesAndFolders(curPath)
+            shutil.rmtree(curPath)
 
 if __name__ == "__main__":
     import gParaSearch
 
-# std module imports
-import numpy as np
-import os
-import re
-import shutil
-import sys
-import time
-import datetime
 
 # gc3 library imports
 import gc3libs
@@ -239,6 +237,26 @@ class gParaSearchParallel(ParallelTaskCollection, paraLoop_fp):
 
     def __init__(self, inParaCombos, iteration, pathToExecutable, pathToStageDir, architecture, baseDir, xVars,
                  solverVerb, problemType, analyzeResults, ctryList, **extra_args):
+        
+        '''
+          Generate a list of tasks and initialize a ParallelTaskCollection with them. 
+          Uses paraLoop class to generate a list of (descriptions, substitutions for the input files). Descriptions are generated from
+          variable names that are hard coded in this method right now. 
+          Uses method generateTaskList to create a list of GPremiumApplication's which are invoked from a list of inputs (appropriately adjusted input files), 
+          the output directory and some further settings for each run. 
+          
+          inParaCombos:      List of tuples defining the parameter combinations.
+          iteration:         Current iteration number. 
+          pathToExecutable:  Path to the executable (the external program to be called). 
+          pathToStageDir:    Root path. Usually os.getcwd()
+          architecture:      32 or 64 bit.
+          baseDir:           Directory in which the input files are located. 
+          xVars:             Names of the x variables. 
+          solverVerb:        Logger verbosity. 
+          problemType:       Forward premium specific flag to determine which case to look at. 
+          analyzeResults:    Function to use to analyze the emerging output. 
+          ctryList:          Forward premium specific list of ctrys to look at. 
+        '''
 
         logger.debug('entering gParaSearchParalell.__init__')
 
@@ -576,7 +594,7 @@ class gParaSearchScript(SessionBasedScript, paraLoop_fp):
         if os.path.isdir(self.params.executable):
             self.params.executable = os.path.join(self.params.executable,
                                                   'forwardPremium')
-        gc3libs.utils.test_file(self.params.executable, os.R_OK|os.X_OK,
+        gc3libs.utils.check_file_access(self.params.executable, os.R_OK|os.X_OK,
                                 gc3libs.exceptions.InvalidUsage)
 
 
