@@ -1,7 +1,7 @@
     #! /usr/bin/env python
 #
 
-# Copyright (C) 2011 University of Zurich. All rights reserved.
+# Copyright (C) 2011  University of Zurich. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,7 +17,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #
-__version__ = '$Revision$'
 __author__ = 'Riccardo Murri <riccardo.murri@uzh.ch>, Benjamin Jonen <benjamin.jonen@bf.uzh.ch>'
 # summary of user-visible changes
 __changelog__ = """
@@ -25,6 +24,7 @@ __changelog__ = """
 """
 __docformat__ = 'reStructuredText'
 
+from __future__ import absolute_import, print_function
 import gc3libs.debug
 import numpy as np
 import re, os, sys
@@ -272,9 +272,9 @@ def str2vals(strIn):
 
 def format_newVal(newVal):
     import re
-    if re.match("^[0-9]*\.[0-9]*$", str(newVal)):
+    if re.match("^[-]*[0-9]*\.[0-9e-]*$", str(newVal)):
 #    if '.' in str(newVal):
-        newValMat = '%.10f' % float(newVal)
+        newValMat = ('%25.15f' % float(newVal)).strip()
     else:
         try:
             # try to convert to integer, and use decimal repr
