@@ -3,7 +3,7 @@
 #   gbenchmark.py -- Front-end script for benchmarking statistical
 #   Business software on a common set of network data.
 #
-#   Copyright (C) 2011, 2012 GC3, University of Zurich
+#   Copyright (C) 2011, 2012  University of Zurich. All rights reserved.
 #
 #   This program is free software: you can redistribute it and/or
 #   modify
@@ -28,8 +28,8 @@ See the output of ``gbenchmark.py --help`` for program usage
 instructions.
 
 Input parameters consists of:
-:param str network map files: Path to folder containing all network files 
-(in .dat format) in the for of: 
+:param str network map files: Path to folder containing all network files
+(in .dat format) in the for of:
 
 ...
 
@@ -42,7 +42,8 @@ Important: All benchmark have to be executed on the same hardware in order to
 provide consistent benchmark results.
 """
 
-__version__ = '2.2.3 version (SVN $Revision$)'
+from __future__ import absolute_import, print_function
+
 # summary of user-visible changes
 __changelog__ = """
   2014-03-14:
@@ -53,7 +54,7 @@ __docformat__ = 'reStructuredText'
 
 
 # run script, but allow GC3Pie persistence module to access classes defined here;
-# for details, see: http://code.google.com/p/gc3pie/issues/detail?id=95
+# for details, see: https://github.com/uzh/gc3pie/issues/95
 if __name__ == "__main__":
     import gbenchmark
     gbenchmark.GbenchmarkScript().run()
@@ -76,7 +77,7 @@ import gc3libs.utils
 from gc3libs.quantity import Memory, kB, MB, MiB, GB, Duration, hours, minutes, seconds
 from gc3libs.workflow import RetryableTask
 
-BENCHMARKS=['infomap-python', 'infomap-cpp', 'infomap-r']
+BENCHMARKS=['infomap-python', 'infomap-cpp', 'infomap-r', 'lprop-python', 'lprop-r', 'lprop-cpp', 'leadingeigenvector-python', 'leadingeigenvector-r', 'leadingeigenvector-cpp', 'edgebetweenness-r', 'fastgreedy-r', 'multilevel-r', 'optimalmodularity-r', 'spinglass-r', 'walktrap-r', 'edgebetweenness-python', 'fastgreedy-python', 'multilevel-python', 'optimalmodularity-python', 'spinglass-python', 'walktrap-python', 'edgebetweenness-cpp', 'fastgreedy-cpp', 'multilevel-cpp', 'optimalmodularity-cpp', 'spinglass-cpp', 'walktrap-cpp']
 
 ## custom application class
 class GbenchmarkApplication(Application):
@@ -84,8 +85,8 @@ class GbenchmarkApplication(Application):
     Custom class to wrap the execution of the R scripts passed in src_dir.
     """
     application_name = 'benchmark'
-    
-    def __init__(self, network_data_file, **extra_args):
+
+    def __init__(self, network_data_file, run_script, **extra_args):
 
         self.output_dir = extra_args['output_dir']
 
@@ -93,18 +94,16 @@ class GbenchmarkApplication(Application):
 
         network_data_filename = os.path.basename(network_data_file)
         inputs[network_data_file] = network_data_filename
+        inputs[run_script] = os.path.basename(run_script)
 
         # adding wrapper main script
         gbenchmark_wrapper_sh = resource_filename(Requirement.parse("gc3pie"),
-                                                  "gc3libs/etc/gbenchmark_%s_wrapper.sh" % extra_args["benchmark_name"])
+                                                  "gc3libs/etc/gbenchmark_wrapper.sh")
 
         inputs[gbenchmark_wrapper_sh] = "gbenchmark_wrapper.sh"
 
-        arguments = "./gbenchmark_wrapper.sh -b %s ./%s" % (extra_args["benchmark_type"], network_data_filename)
+        arguments = "./gbenchmark_wrapper.sh -d -b %s -r %s ./%s " % (extra_args["benchmark_type"], os.path.basename(run_script), network_data_filename)
 
-        # Allocate as many cores as available on the nodes reserved
-        # by the 'benchmark' flavor.
-        # extra_args.setdefault('requested_cores', 8)
         extra_args['requested_cores'] = 8
 
         Application.__init__(
@@ -118,18 +117,19 @@ class GbenchmarkApplication(Application):
             **extra_args)
 
 class GbenchmarkTask(RetryableTask):
-    def __init__(self, network_data_file, **extra_args):
+    def __init__(self, network_data_file, run_script, **extra_args):
         RetryableTask.__init__(
             self,
             # actual computational job
             GbenchmarkApplication(
-                network_data_file, 
+                network_data_file,
+                run_script,
                 **extra_args),
             **extra_args
             )
-        
+
     def retry(self):
-        """ 
+        """
         Task will be retried iif the application crashed
         due to an error within the exeuction environment
         (e.g. VM crash or LRMS kill)
@@ -141,13 +141,13 @@ class GbenchmarkScript(SessionBasedScript):
     """
     Fro each network file (with '.dat' extension) found in the 'input folder',
     GbenchmarkScript generates as many Tasks as 'benchmarks' defined.
-    
+
     The ``gbenchmark`` command keeps a record of jobs (submitted, executed
     and pending) in a session file (set name with the ``-s`` option); at
     each invocation of the command, the status of all recorded jobs is
     updated, output from finished jobs is collected, and a summary table
     of all known jobs is printed.
-    
+
     Options can specify a maximum number of jobs that should be in
     'SUBMITTED' or 'RUNNING' state; ``gbenchmark`` will delay submission of
     newly-created jobs so that this limit is never exceeded.
@@ -157,7 +157,7 @@ class GbenchmarkScript(SessionBasedScript):
         SessionBasedScript.__init__(
             self,
             version = __version__, # module version == script version
-            application = GbenchmarkTask, 
+            application = GbenchmarkTask,
             # only display stats for the top-level policy objects
             # (which correspond to the processed files) omit counting
             # actual applications because their number varies over
@@ -166,11 +166,15 @@ class GbenchmarkScript(SessionBasedScript):
             )
 
     def setup_options(self):
-        self.add_param("-b", "--benchmark", metavar="[STRING]", 
+        self.add_param("-b", "--benchmark", metavar="[STRING]",
                        dest="benchmarks", default="Infomap",
                        help="Comma separated list of benchmarks that " \
                        " should be executed. " \
-                       "Available benchmarks: [infomap].")
+                       "Available benchmarks: [infomap] [lprop].")
+
+        self.add_param("-R", "--run_script", metavar="[STRING]",
+                       dest="run_script", default=None,
+                       help="Execution script for the given benchmark")
 
     def setup_args(self):
 
@@ -181,15 +185,21 @@ class GbenchmarkScript(SessionBasedScript):
         """
         Check validity of input parameters and selected benchmark.
         """
-        
+
         # check args:
         if not os.path.isdir(self.params.network_path):
             raise gc3libs.exceptions.InvalidUsage(
                 "Invalid path to network data: '%s'. File not found"
                 % self.params.network_path)
 
+        # Verify execution script
+        if not os.path.isfile(self.params.run_script):
+            raise gc3libs.exceptions.InvalidUsage(
+                "Invalid path to execution script: '%s'. File not found"
+                % self.params.run_script)
+
         # Verify the selected benchmark
-        _benchmarks = [benchmark.strip().lower() for benchmark in 
+        _benchmarks = [benchmark.strip().lower() for benchmark in
                            self.params.benchmarks.split(',')]
 
         self.benchmarks = []
@@ -230,25 +240,26 @@ class GbenchmarkScript(SessionBasedScript):
                 extra_args['benchmark_type'] = benchmark_type
 
                 extra_args['output_dir'] = self.params.output
-                extra_args['output_dir'] = extra_args['output_dir'].replace('NAME', 
+                extra_args['output_dir'] = extra_args['output_dir'].replace('NAME',
                                                                             os.path.join(benchmark_name,
                                                                                          benchmark_type,
                                                                                          input_file_name))
-                extra_args['output_dir'] = extra_args['output_dir'].replace('SESSION', 
+                extra_args['output_dir'] = extra_args['output_dir'].replace('SESSION',
                                                                             os.path.join(benchmark_name,
-                                                                                         benchmark_type, 
+                                                                                         benchmark_type,
                                                                                          input_file_name))
-                extra_args['output_dir'] = extra_args['output_dir'].replace('DATE', 
+                extra_args['output_dir'] = extra_args['output_dir'].replace('DATE',
                                                                             os.path.join(benchmark_name,
                                                                                          benchmark_type,
                                                                                          input_file_name))
                 extra_args['output_dir'] = extra_args['output_dir'].replace('TIME',
                                                                             os.path.join(benchmark_name,
-                                                                                         benchmark_type, 
+                                                                                         benchmark_type,
                                                                                          input_file_name))
 
                 tasks.append(GbenchmarkTask(
                     input_file,
+                    os.path.abspath(self.params.run_script),
                     **extra_args))
 
         return tasks
