@@ -3,7 +3,7 @@
 #   gnw_measures.py -- Front-end script for evaluating R-based 'grid'
 #   function over a large dataset.
 #
-#   Copyright (C) 2011, 2012 GC3, University of Zurich
+#   Copyright (C) 2011, 2012  University of Zurich. All rights reserved.
 #
 #   This program is free software: you can redistribute it and/or
 #   modify
@@ -29,7 +29,7 @@ instructions.
 Input parameters consists of:
 :param str id_times file: Path to an .csv file containing input data.
 
-Example: 
+Example:
 "id","ts_reference","tf_reference"
 "12802",1663058794000,Inf
 "2617",1666361937000,1666362018000
@@ -48,11 +48,12 @@ XXX: To be clarified:
 . Should be possible to re-run a subset of the initial chunk list
 without re-creating a new session ?
 e.g. adding a new argument accepting chunk ranges (-R 3000:7500)
-This would trigger the re-run of the whole workflow only 
-for lines between 3000 and 7500 
+This would trigger the re-run of the whole workflow only
+for lines between 3000 and 7500
 """
 
-__version__ = '2.2.3 version (SVN $Revision$)'
+from __future__ import absolute_import, print_function
+
 # summary of user-visible changes
 __changelog__ = """
   2013-11-07:
@@ -63,7 +64,7 @@ __docformat__ = 'reStructuredText'
 
 
 # run script, but allow GC3Pie persistence module to access classes defined here;
-# for details, see: http://code.google.com/p/gc3pie/issues/detail?id=95
+# for details, see: https://github.com/uzh/gc3pie/issues/95
 if __name__ == "__main__":
     import gnw_measures
     gnw_measures.GnwScript().run()
@@ -87,11 +88,11 @@ from gc3libs.workflow import RetryableTask
 
 class GnwApplication(Application):
     """
-    Custom class to wrap the execution of the R function over 
+    Custom class to wrap the execution of the R function over
     a chunked ``id_times`` file.
     """
     application_name = 'gnw_measures'
-    
+
     def __init__(self, id_times_filename, **extra_args):
 
         # setup output references
@@ -114,7 +115,7 @@ class GnwApplication(Application):
         else:
             # Use default
             arguments +=  "~/bin/run.R "
-            
+
         # Grid function
         if extra_args.has_key('grid_function'):
             inputs[extra_args['grid_function']] = "./bin/grid.r"
@@ -158,29 +159,29 @@ class GnwTask(RetryableTask):
             self,
             # actual computational job
             GnwApplication(
-                id_times_filename, 
+                id_times_filename,
                 **extra_args),
             **extra_args
             )
 
 class GnwScript(SessionBasedScript):
     """
-    Splits input .csv file into smaller chunks, each of them of size 
+    Splits input .csv file into smaller chunks, each of them of size
     'self.params.chunk_size'.
     Then it submits one execution for each of the created chunked files.
-    
+
     The ``gnw_measures`` command keeps a record of jobs (submitted, executed
     and pending) in a session file (set name with the ``-s`` option); at
     each invocation of the command, the status of all recorded jobs is
     updated, output from finished jobs is collected, and a summary table
     of all known jobs is printed.
-    
+
     Options can specify a maximum number of jobs that should be in
     'SUBMITTED' or 'RUNNING' state; ``gnw_measures`` will delay submission of
     newly-created jobs so that this limit is never exceeded.
 
     Once the processing of all chunked files has been completed, ``gnw_measures``
-    aggregates them into a single larger output file located in 
+    aggregates them into a single larger output file located in
     'self.params.output'.
     """
 
@@ -188,7 +189,7 @@ class GnwScript(SessionBasedScript):
         SessionBasedScript.__init__(
             self,
             version = __version__, # module version == script version
-            application = GnwTask, 
+            application = GnwTask,
             # only display stats for the top-level policy objects
             # (which correspond to the processed files) omit counting
             # actual applications because their number varies over
@@ -200,7 +201,7 @@ class GnwScript(SessionBasedScript):
         self.add_param("-k", "--chunk", metavar="[NUM]", #type=executable_file,
                        dest="chunk_size", default="1000",
                        help="How to split the edges input data set.")
-        
+
         self.add_param("-M", "--master", metavar="[PATH]",
                        dest="driver_script", default=None,
                        help="Location of master driver R script.")
@@ -223,7 +224,7 @@ class GnwScript(SessionBasedScript):
         Check presence of input folder (should contains R scripts).
         path to command_file should also be valid.
         """
-        
+
         # check input arguments
         if not os.path.isfile(self.params.id_times):
             raise gc3libs.exceptions.InvalidUsage(
@@ -243,7 +244,7 @@ class GnwScript(SessionBasedScript):
         tasks = []
 
         for (input_file, index_chunk) in self._generate_chunked_files_and_list(self.params.id_times,
-                                                                              self.params.chunk_size):            
+                                                                              self.params.chunk_size):
             jobname = "gnw_measures-%s" % (str(index_chunk))
 
             extra_args = extra.copy()
@@ -251,21 +252,21 @@ class GnwScript(SessionBasedScript):
             extra_args['index_chunk'] = str(index_chunk)
 
             extra_args['jobname'] = jobname
-            
+
             extra_args['output_dir'] = self.params.output
-            extra_args['output_dir'] = extra_args['output_dir'].replace('NAME', 
+            extra_args['output_dir'] = extra_args['output_dir'].replace('NAME',
                                                                         os.path.join('computation',
                                                                                      jobname))
-            extra_args['output_dir'] = extra_args['output_dir'].replace('SESSION', 
+            extra_args['output_dir'] = extra_args['output_dir'].replace('SESSION',
                                                                         os.path.join('computation',
                                                                                      jobname))
-            extra_args['output_dir'] = extra_args['output_dir'].replace('DATE', 
+            extra_args['output_dir'] = extra_args['output_dir'].replace('DATE',
                                                                         os.path.join('computation',
                                                                                      jobname))
-            extra_args['output_dir'] = extra_args['output_dir'].replace('TIME', 
+            extra_args['output_dir'] = extra_args['output_dir'].replace('TIME',
                                                                         os.path.join('computation',
                                                                                      jobname))
-            
+
 
             if self.params.driver_script:
                 extra_args['driver_script'] = self.params.driver_script
@@ -297,7 +298,7 @@ class GnwScript(SessionBasedScript):
                 if isinstance(task,GnwTask) and task.execution.returncode == 0:
                     try:
                         for line in open(task.output_file):
-                            fout.write(line)    
+                            fout.write(line)
                     except OSError, osx:
                         # Report failure and continue
                         # XXX: Check what would be the correct behaviour
@@ -345,11 +346,11 @@ class GnwScript(SessionBasedScript):
 
             for (i, line) in enumerate(fd):
                 if i % chunk_size == 0:
-                    if fout: 
+                    if fout:
                         fout.close()
                     (handle, self.tmp_filename) = tempfile.mkstemp(dir=chunk_files_dir,
                                                                     prefix=
-                                                                   'gnw_measuresn-', 
+                                                                   'gnw_measuresn-',
                                                                    suffix=
                                                                    "%d.csv" % i)
                     fout = open(self.tmp_filename,'w')
