@@ -2,7 +2,7 @@
 #
 #   gcodeml.py -- Front-end script for submitting multiple CODEML jobs to SMSCG.
 #
-#   Copyright (C) 2010-2012 GC3, University of Zurich
+#   Copyright (C) 2010-2012  University of Zurich. All rights reserved.
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,10 @@ It uses the generic `gc3libs.cmdline.SessionBasedScript` framework.
 
 See the output of ``gcodeml --help`` for program usage instructions.
 """
-__version__ = '2.2.3 version (SVN $Revision$)'
+
+from __future__ import absolute_import, print_function
+
+__version__ = '2.0.5'
 # summary of user-visible changes
 __changelog__ = """
   2013-02-18:
@@ -60,7 +63,7 @@ __docformat__ = 'reStructuredText'
 
 
 # run script, but allow GC3Pie persistence module to access classes defined here;
-# for details, see: http://code.google.com/p/gc3pie/issues/detail?id=95
+# for details, see: https://github.com/uzh/gc3pie/issues/95
 if __name__ == '__main__':
     import gcodeml
     gcodeml.GCodemlScript().run()
@@ -166,7 +169,7 @@ of newly-created jobs so that this limit is never exceeded.
         if self.params.codeml is not None:
             if not os.path.isabs(self.params.codeml):
                 self.params.codeml = os.path.abspath(self.params.codeml)
-            gc3libs.utils.test_file(self.params.codeml, os.R_OK|os.X_OK)
+            gc3libs.utils.check_file_access(self.params.codeml, os.R_OK|os.X_OK)
 
         ## collect input directories/files
         def contain_ctl_files(paths):
