@@ -2,7 +2,7 @@
 #
 #   ggamess.py -- Front-end script for submitting multiple GAMESS jobs to SMSCG.
 #
-#   Copyright (C) 2010-2012 GC3, University of Zurich
+#   Copyright (C) 2010-2012  University of Zurich. All rights reserved.
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,6 @@ It uses the generic `gc3libs.cmdline.SessionBasedScript` framework.
 
 See the output of ``ggamess --help`` for program usage instructions.
 """
-__version__ = '2.2.3 version (SVN $Revision$)'
 # summary of user-visible changes
 __changelog__ = """
   2012-09-11:
@@ -42,6 +41,7 @@ __docformat__ = 'reStructuredText'
 
 
 # stdlib imports
+from __future__ import absolute_import, print_function
 import os
 import sys
 
@@ -52,7 +52,7 @@ from gc3libs.cmdline import SessionBasedScript, existing_file
 
 
 # run script, but allow GC3Pie persistence module to access classes defined here;
-# for details, see: http://code.google.com/p/gc3pie/issues/detail?id=95
+# for details, see: https://github.com/uzh/gc3pie/issues/95
 if __name__ == '__main__':
     import ggamess
     ggamess.GGamessScript().run()
