@@ -2,7 +2,7 @@
 #
 #   gmhc_coev.py -- Front-end script for submitting multiple `MHC_coev` jobs to SMSCG.
 #
-#   Copyright (C) 2011-2012 GC3, University of Zurich
+#   Copyright (C) 2011-2012  University of Zurich. All rights reserved.
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,9 @@ It uses the generic `gc3libs.cmdline.SessionBasedScript` framework.
 
 See the output of ``gmhc_coev --help`` for program usage instructions.
 """
-__version__ = '2.2.3 version (SVN $Revision$)'
+
+from __future__ import absolute_import, print_function
+
 # summary of user-visible changes
 __changelog__ = """
   2011-07-22:
@@ -36,7 +38,7 @@ __docformat__ = 'reStructuredText'
 
 
 # run script, but allow GC3Pie persistence module to access classes defined here;
-# for details, see: http://code.google.com/p/gc3pie/issues/detail?id=95
+# for details, see: https://github.com/uzh/gc3pie/issues/95
 if __name__ == '__main__':
     import gmhc_coev
     gmhc_coev.GMhcCoevScript().run()
@@ -56,7 +58,7 @@ import time
 import gc3libs
 from gc3libs import Application, Run, Task
 from gc3libs.cmdline import SessionBasedScript
-from gc3libs.compat._collections import defaultdict
+from collections import defaultdict
 from gc3libs.quantity import Memory, kB, MB, GB, Duration, hours, minutes, seconds
 from gc3libs.workflow import SequentialTaskCollection
 
