@@ -2,7 +2,7 @@
 #
 #   gdocking.py -- Front-end script for submitting ROSETTA `docking_protocol` jobs to SMSCG.
 #
-#   Copyright (C) 2010-2012 GC3, University of Zurich
+#   Copyright (C) 2010-2012  University of Zurich. All rights reserved.
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,9 @@ It uses the generic `gc3libs.cmdline.SessionBasedScript` framework.
 
 See the output of ``gdocking --help`` for program usage instructions.
 """
-__version__ = '2.2.3 version (SVN $Revision$)'
+
+from __future__ import absolute_import, print_function
+
 __author__ = 'Riccardo Murri <riccardo.murri@uzh.ch>'
 # summary of user-visible changes
 __changelog__ = """
@@ -86,7 +88,7 @@ __docformat__ = 'reStructuredText'
 
 
 # workaround for Issue 95,
-# see: http://code.google.com/p/gc3pie/issues/detail?id=95
+# see: https://github.com/uzh/gc3pie/issues/95
 if __name__ == "__main__":
     import gdocking
     gdocking.GDockingScript().run()
@@ -211,13 +213,13 @@ of newly-created jobs so that this limit is never exceeded.
             version = __version__, # module version == script version
             # Use fully-qualified name for class,
             # to allow GC3Utils to unpickle job instances
-            # see: http://code.google.com/p/gc3pie/issues/detail?id=95
+            # see: https://github.com/uzh/gc3pie/issues/95
             application = GDockingApplication,
             input_filename_pattern = '*.pdb'
             )
 
     def setup_options(self):
-        self.add_param("-f", "--flags-file", dest="flags_file", 
+        self.add_param("-f", "--flags-file", dest="flags_file",
                        default=os.path.join(gc3libs.Default.RCDIR, 'docking_protocol.flags'),
                        metavar="PATH",
                        help="Pass the specified flags file to Rosetta 'docking_protocol'"
@@ -263,7 +265,6 @@ of newly-created jobs so that this limit is never exceeded.
         except Exception, ex:
             gc3libs.log.error("Invalid flags file `%s`: please supply a valid flags file by adding the option `-f PATH`" % self.params.flags_file)
             raise ex
-        
+
         self.extra['flags_file'] = os.path.abspath(self.params.flags_file)
         self.log.info("Using flags file '%s'", self.extra['flags_file'])
-
