@@ -1,8 +1,12 @@
 #! /usr/bin/env python
-#
+
+"""
+Simple interface to the CODEML application.
+"""
+
 #   codeml.py -- Simple interface to the CODEML application
 #
-#   Copyright (C) 2010, 2011, 2012 GC3, University of Zurich
+#   Copyright (C) 2010, 2011, 2012, 2019  University of Zurich. All rights reserved.
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -16,11 +20,11 @@
 #
 #   You should have received a copy of the GNU General Public License
 #   along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#
-"""
-Simple interface to the CODEML application.
-"""
-__version__ = '2.2.3 (SVN $Revision$)'
+
+
+from __future__ import absolute_import, print_function, unicode_literals
+from builtins import str
+from builtins import range
 __changelog__ = """
 Summary of user-visible changes:
 * 16-08-2011 AM: Extract aln_info from the .phy file and record it
@@ -47,9 +51,10 @@ import gc3libs
 import gc3libs.utils
 
 
-## The CODEML application
+# The CODEML application
 
 class CodemlApplication(gc3libs.Application):
+
     """
     Run a CODEML job with the specified '.ctl' files.
 
@@ -74,10 +79,10 @@ class CodemlApplication(gc3libs.Application):
 
         # need to send the PERL driver script, and the binary only
         # if we're not using the RTE
-        inputs = { codeml_pl:'codeml.pl' }
+        inputs = {codeml_pl: 'codeml.pl'}
 
         # include comdel in the list of executables
-        if extra_args.has_key('executables'):
+        if 'executables' in extra_args:
             extra_args['executables'].append('codeml.pl')
         else:
             extra_args['executables'] = ['codeml.pl']
@@ -88,19 +93,18 @@ class CodemlApplication(gc3libs.Application):
             if 'tags' in extra_args:
                 extra_args['tags'].append(rte)
             else:
-                extra_args['tags'] = [ rte ]
+                extra_args['tags'] = [rte]
         else:
             # use provided binary
             inputs[codeml] = 'codeml'
 
-
         # set result dir: where the expected output files
-        # will be copied as part of 'terminated' 
+        # will be copied as part of 'terminated'
         if 'result_dir' in extra_args:
             self.result_dir = extra_args['result_dir']
 
         # output file paths are read from the '.ctl' file below
-        outputs = [ ]
+        outputs = []
         # for each '.ctl' file, extract the referenced "seqfile" and
         # "treefile" and add them to the input list
         for ctl in ctls:
@@ -108,7 +112,7 @@ class CodemlApplication(gc3libs.Application):
                 # try getting the seqfile/treefile path before we
                 # append the '.ctl' file to inputs; if they cannot be
                 # found, we do not append the '.ctl' either...
-                for (key, path) in CodemlApplication.aux_files(ctl).items():
+                for (key, path) in list(CodemlApplication.aux_files(ctl).items()):
                     if key in ['seqfile', 'treefile'] and path not in inputs:
                         inputs[path] = os.path.basename(path)
 
@@ -118,11 +122,11 @@ class CodemlApplication(gc3libs.Application):
                             fd = open(path)
                             aln_infos = fd.readline().strip().split()
                             self.aln_info = {
-                                'n_seq'   : int(aln_infos[0]),
-                                'aln_len' : int(aln_infos[1]),
-                                }
+                                'n_seq': int(aln_infos[0]),
+                                'aln_len': int(aln_infos[1]),
+                            }
                             fd.close()
-                        except Exception, ex:
+                        except Exception as ex:
                             gc3libs.log.warning(
                                 "Unable to parse `n_seq` and `aln_len` values"
                                 " from `.phy` file `%s`: %s", path, str(ex))
@@ -138,24 +142,26 @@ class CodemlApplication(gc3libs.Application):
             # if the phy/nwk files are not found, `aux_files` raises
             # an exception; catch it here and ignore the '.ctl' file
             # as well.
-            except RuntimeError, ex:
-                gc3libs.log.warning("Ignoring input file '%s':"
-                                    " cannot find seqfile and/or treefile referenced in it: %s"
-                                    % (ctl, str(ex)))
+            except RuntimeError as ex:
+                gc3libs.log.warning(
+                    "Ignoring input file '%s':"
+                    " cannot find seqfile and/or treefile referenced in it: %s" %
+                    (ctl, str(ex)))
         gc3libs.Application.__init__(
             self,
-            arguments = ["./codeml.pl"] + [ os.path.basename(ctl) for ctl in ctls ],
-            inputs = inputs,
-            outputs = outputs,
-            stdout = 'codeml.stdout.txt',
-            stderr = 'codeml.stderr.txt',
+            arguments=["./codeml.pl"] +
+            [os.path.basename(ctl) for ctl in ctls],
+            inputs=inputs,
+            outputs=outputs,
+            stdout='codeml.stdout.txt',
+            stderr='codeml.stderr.txt',
             # an estimation of wall-clock time requirements can be
             # derived from the '.phy' input file, use it to set the
             # `required_walltime` attribute, so we do not risk jobs
             # being killed because they exceed allotted running time
-            #required_walltime = ...,
+            # required_walltime = ...,
             **extra_args
-            )
+        )
 
         # these attributes will get their actual value after
         # `terminated()` has run; pre-set them here to an invalid
@@ -168,13 +174,9 @@ class CodemlApplication(gc3libs.Application):
         self.valid = [None] * len(ctls)
         self.time_used = [None] * len(ctls)
 
-
-
-
     # split a line 'key = value' around the middle '=' and ignore spaces
-    _assignment_re = re.compile('\s* = \s*', re.X)
-    _aux_file_keys = [ 'seqfile', 'treefile', 'outfile' ]
-
+    _assignment_re = re.compile(r'\s* = \s*', re.X)
+    _aux_file_keys = ['seqfile', 'treefile', 'outfile']
 
     # aux function to get thw seqfile and treefile paths
     @staticmethod
@@ -184,12 +186,13 @@ class CodemlApplication(gc3libs.Application):
         the '.ctl' file given as arguments.
         """
         dirname = os.path.dirname(ctl_path)
+
         def abspath(filename):
             if os.path.isabs(filename):
                 return filename
             else:
                 return os.path.join(dirname, filename)
-        result = { }
+        result = {}
         ctl = open(ctl_path, 'r')
         for line in ctl.readlines():
             # remove comments (from '*' to end-of line)
@@ -199,10 +202,11 @@ class CodemlApplication(gc3libs.Application):
             # ignore empty lines
             if len(line) == 0:
                 continue
-            key, value = CodemlApplication._assignment_re.split(line, maxsplit=1)
+            key, value = CodemlApplication._assignment_re.split(
+                line, maxsplit=1)
             if key not in CodemlApplication._aux_file_keys:
                 continue
-            elif key in [ 'seqfile', 'treefile' ]:
+            elif key in ['seqfile', 'treefile']:
                 result[key] = abspath(value)
             elif key == 'outfile':
                 result[key] = value
@@ -215,9 +219,9 @@ class CodemlApplication(gc3libs.Application):
         # ``treefile = ...`` lines were not found; signal this to the
         # caller by raising an exception
         ctl.close()
-        raise RuntimeError("Could not extract path to seqfile and/or treefile from '%s'"
-                           % ctl_path)
-
+        raise RuntimeError(
+            "Could not extract path to seqfile and/or treefile from '%s'" %
+            ctl_path)
 
     @staticmethod
     def parse_output_file(path):
@@ -237,20 +241,22 @@ class CodemlApplication(gc3libs.Application):
                 time_used_found = True
                 break
         if time_used_found:
-            return (hours*3600 + minutes*60 + seconds)
+            return (hours * 3600 + minutes * 60 + seconds)
         else:
             return 'invalid'
 
-
     # stdout starts with `HOST: ...` and `CPU: ` lines, but there's a
     # variable number of spaces so we need a regexp to match
     _KEY_VALUE_SEP = re.compile(r':\s*')
 
-    _TIME_USED_RE = re.compile(r'Time \s+ used:\s+ ((?P<hours>[0-9]*):)?(?P<minutes>[0-9]+):(?P<seconds>[0-9]+)', re.X)
+    _TIME_USED_RE = re.compile(
+        r'Time \s+ used:\s+ ((?P<hours>[0-9]*):)?(?P<minutes>[0-9]+):(?P<seconds>[0-9]+)',
+        re.X)
     _H_WHICH_RE = re.compile(r'H(?P<n>[01]).mlc')
 
     # perform the post-processing and set exit code.
-    # This is the final result verification of a single codeml H0/H1 tuple (job)
+    # This is the final result verification of a single codeml H0/H1 tuple
+    # (job)
     def terminated(self):
         """
         Set the exit code of a `CodemlApplication` job by inspecting its
@@ -298,8 +304,12 @@ class CodemlApplication(gc3libs.Application):
             self.execution.exitcode = 127
             return
 
-        outputs = [ os.path.join(download_dir, filename)
-                    for filename in fnmatch.filter(os.listdir(download_dir), '*.mlc') ]
+        outputs = [
+            os.path.join(
+                download_dir,
+                filename) for filename in fnmatch.filter(
+                os.listdir(download_dir),
+                '*.mlc')]
         if len(outputs) == 0:
             # no output retrieved, did ``codeml`` run at all?
             self.execution.exitcode = 127
@@ -314,8 +324,9 @@ class CodemlApplication(gc3libs.Application):
                 if match:
                     n = int(match.group('n'))
                 else:
-                    gc3libs.log.debug("Output file '%s' does not match pattern 'H*.mlc' -- ignoring.")
-                    continue # with next output_path
+                    gc3libs.log.debug(
+                        "Output file '%s' does not match pattern 'H*.mlc' -- ignoring.")
+                    continue  # with next output_path
                 duration = CodemlApplication.parse_output_file(output_path)
                 if duration == 'no file':
                     self.exists[n] = False
@@ -335,21 +346,27 @@ class CodemlApplication(gc3libs.Application):
             import shutil
             for output_path in outputs:
                 try:
-                    shutil.copy(output_path, 
+                    shutil.copy(output_path,
                                 os.path.join(self.result_dir,
                                              os.path.basename(output_path)))
                     os.remove(output_path)
-                except Exception, ex:
-                    gc3libs.log.error("Failed while transferring output file " +
-                                      "%s " % output_path +
-                                      "to result folder %s. " % self.result_dir +
-                                      "Error type %s. Message %s. " 
-                                      % (type(ex),str(ex)))
+                except Exception as ex:
+                    gc3libs.log.error(
+                        "Failed while transferring output file " +
+                        "%s " %
+                        output_path +
+                        "to result folder %s. " %
+                        self.result_dir +
+                        "Error type %s. Message %s. " %
+                        (type(ex),
+                         str(ex)))
                     failed += 1
                     continue
         else:
-            gc3libs.log.warning("No 'result_dir' set. Leaving results in original" +
-                                " output folder %s" % self.output_dir)
+            gc3libs.log.warning(
+                "No 'result_dir' set. Leaving results in original" +
+                " output folder %s" %
+                self.output_dir)
 
         # if output files parsed OK, then override the exit code and
         # mark the job as successful
@@ -363,9 +380,11 @@ class CodemlApplication(gc3libs.Application):
             for line in stdout_file:
                 line = line.strip()
                 if line.startswith('HOST:'):
-                    tag, self.hostname = CodemlApplication._KEY_VALUE_SEP.split(line, maxsplit=1)
+                    tag, self.hostname = CodemlApplication._KEY_VALUE_SEP.split(
+                        line, maxsplit=1)
                 elif line.startswith('CPU:'):
-                    tag, self.cpuinfo = CodemlApplication._KEY_VALUE_SEP.split(line, maxsplit=1)
+                    tag, self.cpuinfo = CodemlApplication._KEY_VALUE_SEP.split(
+                        line, maxsplit=1)
                     break
             stdout_file.close()
 
