@@ -1,9 +1,10 @@
 #! /usr/bin/env python
-#
+
 """
 Specialized support for computational jobs running simple demo.
 """
-# Copyright (C) 2009-2012 GC3, University of Zurich. All rights reserved.
+
+# Copyright (C) 2009-2012  University of Zurich. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -16,15 +17,14 @@ Specialized support for computational jobs running simple demo.
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-#
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+from __future__ import absolute_import, print_function, unicode_literals
+from builtins import str
 __docformat__ = 'reStructuredText'
-__version__ = '2.2.3 version (SVN $Revision: 4001 $)'
 
 
 import os
-import os.path
 from pkg_resources import Requirement, resource_filename
 
 import gc3libs
@@ -34,6 +34,7 @@ from gc3libs.quantity import GB, hours
 
 
 class Square(gc3libs.Application):
+
     """
     Square class, takes a filename containing a list of integer to be squared.
     writes an output containing the square of each of them
@@ -55,21 +56,27 @@ class Square(gc3libs.Application):
         # extra_args.setdefault('stderr', 'stderr.txt')
 
         gc3libs.Application.__init__(self,
-                                     arguments = ["/usr/bin/expr", str(x), "*", str(x)],
-                                     inputs = [],
-                                     outputs = [],
-                                     output_dir = None,
-                                     stdout = "stdout.txt",
-                                     stderr = "stderr.txt",
-                                     # set computational requirements. XXX this is mandatory, thus probably should become part of the Application's signature
-                                     requested_memory = 1*GB,
-                                     requested_cores = 1,
-                                     requested_walltime = 1*hours,
+                                     arguments=[
+                                         "/usr/bin/expr",
+                                         str(x),
+                                         "*",
+                                         str(x)],
+                                     inputs=[],
+                                     outputs=[],
+                                     output_dir=None,
+                                     stdout="stdout.txt",
+                                     stderr="stderr.txt",
+                                     # set computational requirements. XXX this
+                                     # is mandatory, thus probably should
+                                     # become part of the Application's
+                                     # signature
+                                     requested_memory=1 * GB,
+                                     requested_cores=1,
+                                     requested_walltime=1 * hours,
                                      )
 
 
-
-## main: run tests
+# main: run tests
 
 if "__main__" == __name__:
     import doctest
