@@ -1,9 +1,10 @@
 #! /usr/bin/env python
-#
+
 """
 Specialized support for computational jobs running programs in the Rosetta suite.
 """
-# Copyright (C) 2009-2012 GC3, University of Zurich. All rights reserved.
+
+# Copyright (C) 2009-2012  University of Zurich. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -16,16 +17,15 @@ Specialized support for computational jobs running programs in the Rosetta suite
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-#
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+from __future__ import absolute_import, print_function, unicode_literals
+from builtins import str
 __docformat__ = 'reStructuredText'
-__version__ = '2.2.3 version (SVN $Revision: 4001 $)'
 
 
 import gc3libs
 import gc3libs.application
-from gc3libs.exceptions import *
 import os
 import os.path
 from pkg_resources import Requirement, resource_filename
@@ -33,6 +33,7 @@ import tarfile
 
 
 class RosettaApplication(gc3libs.Application):
+
     """
     Specialized `Application` object to submit one run of a single
     application in the Rosetta suite.
@@ -80,13 +81,16 @@ class RosettaApplication(gc3libs.Application):
         _outputs = [
             self.__protocol + '.log',
             self.__protocol + '.tar.gz'
-            ]
+        ]
         if outputs:
-            _arguments = ['--tar', str.join(' ', [ str(o) for o in outputs ])]
+            _arguments = ['--tar', ' '.join(['{0}'.format(o) for o in outputs])]
         else:
             _arguments = ['--tar', '*.pdb *.sc *.fasc']
 
-        _inputs = gc3libs.Application._io_spec_to_dict(gc3libs.url.UrlKeyDict, inputs, True)
+        _inputs = gc3libs.Application._io_spec_to_dict(
+            gc3libs.url.UrlKeyDict,
+            inputs,
+            True)
         if flags_file:
             _inputs[flags_file] = self.__protocol + '.flags'
             # the `rosetta.sh` driver will do this automatically:
@@ -104,22 +108,21 @@ class RosettaApplication(gc3libs.Application):
         if 'tags' in extra_args:
             extra_args['tags'].append(application_release)
         else:
-            extra_args['tags'] = [ application_release ]
+            extra_args['tags'] = [application_release]
 
-        extra_args.setdefault('stdout', application+'.stdout.txt')
-        extra_args.setdefault('stderr', application+'.stderr.txt')
+        extra_args.setdefault('stdout', application + '.stdout.txt')
+        extra_args.setdefault('stderr', application + '.stderr.txt')
 
         rosetta_sh = self.__protocol + '.sh'
         _inputs[src_rosetta_sh] = rosetta_sh
 
         gc3libs.Application.__init__(
             self,
-            arguments = [ ("./%s" % rosetta_sh) ] + _arguments,
-            inputs = _inputs,
-            outputs = _outputs,
+            arguments=[("./%s" % rosetta_sh)] + _arguments,
+            inputs=_inputs,
+            outputs=_outputs,
             **extra_args)
 
-
     def terminated(self):
         """
         Extract output files from the tar archive created by the
@@ -134,19 +137,23 @@ class RosettaApplication(gc3libs.Application):
                     tar_file = tarfile.open(tar_file_name, 'r:gz')
                     tar_file.extractall(path=output_dir)
                     os.remove(tar_file_name)
-                except tarfile.TarError, ex:
-                    gc3libs.log.error("Error extracting output from archive '%s': %s: %s"
-                                      % (tar_file_name, ex.__class__.__name__, str(ex)))
+                except tarfile.TarError as ex:
+                    gc3libs.log.error(
+                        "Error extracting output from archive '%s': %s: %s" %
+                        (tar_file_name, ex.__class__.__name__, str(ex)))
             else:
-                gc3libs.log.error("Could not extract output archive '%s':"
-                                  " format not handled by Python 'tarfile' module"
-                                  % tar_file_name)
+                gc3libs.log.error(
+                    "Could not extract output archive '%s':"
+                    " format not handled by Python 'tarfile' module" %
+                    tar_file_name)
         else:
-            gc3libs.log.error("Could not find output archive '%s' for Rosetta job"
-                              % tar_file_name)
+            gc3libs.log.error(
+                "Could not find output archive '%s' for Rosetta job" %
+                tar_file_name)
 
 
 class RosettaDockingApplication(RosettaApplication):
+
     """
     Specialized `Application` class for executing a single run of the
     Rosetta "docking_protocol" application.
@@ -166,27 +173,26 @@ class RosettaDockingApplication(RosettaApplication):
             native_file_path = pdb_file_path
         RosettaApplication.__init__(
             self,
-            application = 'docking_protocol',
-            application_release = application_release,
-            inputs = [
+            application='docking_protocol',
+            application_release=application_release,
+            inputs=[
                 pdb_file_path,
                 native_file_path,
-                ],
-            outputs = [
-                ],
-            flags_file = flags_file,
-            arguments = [
+            ],
+            outputs=[
+            ],
+            flags_file=flags_file,
+            arguments=[
                 "-in:file:s", os.path.basename(pdb_file_path),
                 "-in:file:native", os.path.basename(native_file_path),
                 "-out:file:o", pdb_file_name_sans,
                 "-out:nstruct", number_of_decoys_to_create,
-                ] + extra_args.pop('arguments', []),
-            output_dir = extra_args.pop('output_dir', pdb_file_dir),
+            ] + extra_args.pop('arguments', []),
+            output_dir=extra_args.pop('output_dir', pdb_file_dir),
             **extra_args)
 
 
-
-## main: run tests
+# main: run tests
 
 if "__main__" == __name__:
     import doctest
