@@ -1,8 +1,10 @@
 #! /usr/bin/env python
+
 """
 Interface to different resource management systems for the GC3Libs.
 """
-# Copyright (C) 2009-2014 GC3, University of Zurich. All rights reserved.
+
+# Copyright (C) 2009-2015  University of Zurich. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -15,22 +17,24 @@ Interface to different resource management systems for the GC3Libs.
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-#
-__docformat__ = 'reStructuredText'
-__version__ = '2.2.3 version (SVN $Revision: 4001 $)'
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+from __future__ import absolute_import, print_function, unicode_literals
+from builtins import str
+__docformat__ = 'reStructuredText'
+
+
+from functools import wraps
 
 import gc3libs
-from gc3libs.compat.functools import wraps
 import gc3libs.exceptions
-from gc3libs.quantity import Memory, kB, MB, GB
-from gc3libs.quantity import Duration, hours, minutes, seconds
+from gc3libs.quantity import Memory
+from gc3libs.quantity import Duration
 import gc3libs.utils
 
 
 class LRMS(gc3libs.utils.Struct):
+
     """Base class for interfacing with a computing resource.
 
     The following construction parameters are also set as instance
@@ -102,6 +106,7 @@ class LRMS(gc3libs.utils.Struct):
     ===================  =====
 
     """
+
     def __init__(self, name,
                  architecture, max_cores, max_cores_per_job,
                  max_memory_per_core, max_walltime, auth=None,
@@ -146,7 +151,20 @@ class LRMS(gc3libs.utils.Struct):
         @wraps(fn)
         def wrapper(self, *args, **kwargs):
             if self._auth_fn is not None:
-                self._auth_fn()
+                try:
+                    self._auth_fn()
+                except Exception as err:
+                    # log error and disable resource - if the "*Auth"
+                    # object cannot be instanciated, there is not much
+                    # we can do...
+                    gc3libs.log.warning(
+                        "Problems initializing authentication backend"
+                        " for resource '%s': %s.", self.name, err)
+                    gc3libs.log.error(
+                        "Resource '%s' will be disabled"
+                        " because of authentication problems.",
+                        self.name)
+                    self.enabled = False
             return fn(self, *args, **kwargs)
         return wrapper
 
@@ -183,7 +201,8 @@ class LRMS(gc3libs.utils.Struct):
             "Abstract method `LRMS.get_resource_status()` called "
             "- this should have been defined in a derived class.")
 
-    def get_results(self, job, download_dir, overwrite=False, changed_only=True):
+    def get_results(self, job, download_dir,
+                    overwrite=False, changed_only=True):
         """
         Retrieve job output files into local directory `download_dir`.
 
@@ -301,8 +320,7 @@ class LRMS(gc3libs.utils.Struct):
             "- this should have been defined in a derived class.")
 
 
-## main: run tests
-
+# main: run tests
 if "__main__" == __name__:
     import doctest
     doctest.testmod(name="__init__",
