@@ -1,9 +1,10 @@
 #! /usr/bin/env python
-#
+
 """
 Job control on SGE clusters (possibly connecting to the front-end via SSH).
 """
-# Copyright (C) 2009-2013 GC3, University of Zurich. All rights reserved.
+
+# Copyright (C) 2009-2014, 2016  University of Zurich. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -16,40 +17,33 @@ Job control on SGE clusters (possibly connecting to the front-end via SSH).
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-#
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+from __future__ import absolute_import, print_function, unicode_literals
+from builtins import str
 __docformat__ = 'reStructuredText'
-__version__ = '2.2.3 version (SVN $Revision: 4001 $)'
 
 
 # stdlib imports
 import datetime
-from getpass import getuser
 import math
-import os
-import posixpath
-import random
 import re
-import sys
-import tempfile
-import time
 
-from gc3libs.compat._collections import defaultdict
+from collections import defaultdict
 
 # GC3Pie imports
 from gc3libs import log, Run
 from gc3libs.backends import LRMS
 import gc3libs.exceptions
 from gc3libs.quantity import Memory, kB, MB, GB
-from gc3libs.quantity import Duration, hours, minutes, seconds
+from gc3libs.quantity import seconds
 import gc3libs.utils
-from gc3libs.utils import same_docstring_as
-import transport
-import batch
+from gc3libs.utils import (same_docstring_as, sh_quote_safe_cmdline,
+                           sh_quote_unsafe_cmdline)
+from . import batch
 
 
-## auxiliary functions
+# auxiliary functions
 
 def _int_floor(val):
     """Return `val` rounded to nearest integer towards 0."""
@@ -60,8 +54,8 @@ def _to_duration(val):
     """Convert a floating point number of seconds to a
     `gc3libs.quantity.Duration` value."""
     try:
-        return float(val)*seconds
-    except Exception, err:
+        return float(val) * seconds
+    except Exception as err:
         gc3libs.log.warning(
             "Grid Engine backend:"
             " Cannot interpret '%s' as a duration (time unit: seconds):"
@@ -76,45 +70,46 @@ def _to_memory(val):
     try:
         unit = val[-1]
         if unit in ['G', 'g']:
-            return float(val[:-2])*GB
+            return float(val[:-2]) * GB
         elif unit in ['M', 'm']:
-            return float(val[:-2])*MB
+            return float(val[:-2]) * MB
         elif unit in ['K', 'k']:
-            return float(val[:-2])*kB
+            return float(val[:-2]) * kB
         else:
             # SGE's default is bytes
-            return float(val)*Memory.B
-    except Exception, err:
+            return float(val) * Memory.B
+    except Exception:
         gc3libs.log.warning("Grid Engine backend: Cannot interpret '%s' "
                             "as a MEMORY value.", val)
         return None
 
+
 # `_convert` is a `dict` instance, mapping key names to functions
 # that parse a value from a string into a Python native type.
 _convert = {
-    'slots':          int,
-    'slots_used':     int,
-    'slots_resv':     int,
-    'slots_total':    int,
-    'load_avg':       float,
-    'load_short':     float,
-    'load_medium':    float,
-    'load_long':      float,
-    'np_load_avg':    float,
-    'np_load_short':  float,
+    'slots': int,
+    'slots_used': int,
+    'slots_resv': int,
+    'slots_total': int,
+    'load_avg': float,
+    'load_short': float,
+    'load_medium': float,
+    'load_long': float,
+    'np_load_avg': float,
+    'np_load_short': float,
     'np_load_medium': float,
-    'np_load_long':   float,
-    'num_proc':       _int_floor,  # SGE considers `num_proc` a
-                                   # floating-point value...
-    'swap_free':      gc3libs.utils.to_bytes,
-    'swap_total':     gc3libs.utils.to_bytes,
-    'swap_used':      gc3libs.utils.to_bytes,
-    'mem_free':       gc3libs.utils.to_bytes,
-    'mem_used':       gc3libs.utils.to_bytes,
-    'mem_total':      gc3libs.utils.to_bytes,
-    'virtual_free':   gc3libs.utils.to_bytes,
-    'virtual_used':   gc3libs.utils.to_bytes,
-    'virtual_total':  gc3libs.utils.to_bytes,
+    'np_load_long': float,
+    'num_proc': _int_floor,  # SGE considers `num_proc` a
+    # floating-point value...
+    'swap_free': gc3libs.utils.to_bytes,
+    'swap_total': gc3libs.utils.to_bytes,
+    'swap_used': gc3libs.utils.to_bytes,
+    'mem_free': gc3libs.utils.to_bytes,
+    'mem_used': gc3libs.utils.to_bytes,
+    'mem_total': gc3libs.utils.to_bytes,
+    'virtual_free': gc3libs.utils.to_bytes,
+    'virtual_used': gc3libs.utils.to_bytes,
+    'virtual_total': gc3libs.utils.to_bytes,
 }
 
 
@@ -126,15 +121,28 @@ def _parse_value(key, value):
 
 
 def _parse_asctime(val):
-    try:
-        # XXX: replace with datetime.strptime(...) in Python 2.5+
-        return datetime.datetime(
-            *(time.strptime(val, '%a %b %d %H:%M:%S %Y')[0:6]))
-    except Exception, err:
-        gc3libs.log.error(
-            "Cannot parse '%s' as a SGE-format time stamp: %s: %s",
-            val, err.__class__.__name__, str(err))
-        return None
+    """
+    Parse a timestamp string output by GE and return a Python `datetime` object.
+    """
+    # while the format of asctime() is strictly mandated by POSIX and
+    # the ISO C standard, Univa GE changed the date/time printing in
+    # `qacct -j` to use the format of `sge_ctime()` which hard-code a
+    # MM/DD/YYYY representation.  Yet another representation (ISO
+    # 8601) is given in XML output... so try them all, one at a time
+    # hoping one succeeds!
+    for fmt in [
+            '%a %b %d %H:%M:%S %Y',  # standard asctime() format
+            '%m/%d/%Y %H:%M:%S',     # sge_ctime()
+            '%m/%d/%Y %H:%M:%S.%f',  # sge_ctime() including milliseconds
+            '%Y-%m-%dT%H:%M:%S',     # ISO 8601 / sge_ctimeXML()
+    ]:
+        try:
+            return datetime.datetime.strptime(val, fmt)
+        except ValueError:
+            pass  # try next format
+    gc3libs.log.error(
+        "Cannot parse '%s' as a SGE-format time stamp", val)
+    return None
 
 
 def parse_qstat_f(qstat_output):
@@ -143,7 +151,7 @@ def parse_qstat_f(qstat_output):
     and return a `dict` instance, mapping each queue name to its attributes.
     """
     # a job report line starts with a numeric job ID
-    _job_line_re = re.compile(r'^[0-9]+ \s+', re.X)
+    # _job_line_re = re.compile(r'^[0-9]+ \s+', re.X)
     # queue report header line starts with queuename@hostname
     _queue_header_re = re.compile(
         r'^([a-z0-9\._-]+)@([a-z0-9\.-]+) \s+ ([BIPCTN]+) '
@@ -221,8 +229,8 @@ def compute_nr_of_slots(qstat_output):
     def dict_with_zero_initializer():
         return defaultdict(zero_initializer)
     result = defaultdict(dict_with_zero_initializer)
-    for q in qstat.iterkeys():
-        for host in qstat[q].iterkeys():
+    for q in qstat.keys():
+        for host in qstat[q].keys():
             r = result[host]
             s = qstat[q][host]
             r['total'] = max(s['slots_total'], r['total'])
@@ -231,7 +239,7 @@ def compute_nr_of_slots(qstat_output):
     # compute available slots by subtracting the number of
     # used+reserved from the total
     g = result['global']
-    for host in result.iterkeys():
+    for host in result.keys():
         r = result[host]
         r['available'] = r['total'] - r['unavailable']
         # update cluster-wide ('global') totals
@@ -253,14 +261,14 @@ def parse_qhost_f(qhost_output):
         n += 1
         if n < 3:
             continue
-        # property lines start with TAB
-        if line.startswith(' '):
+        if not line.startswith(' '):
+            # host lines begin at column 0
+            hostname = line.split(' ')[0]
+        else:
+            # property lines start with TAB
             key, value = line.split('=')
             ignored, key = key.split(':')
             result[hostname][key] = _parse_value(key, value)
-        # host lines begin at column 0
-        else:
-            hostname = line.split(' ')[0]
     return result
 
 
@@ -341,6 +349,7 @@ Regex for extracting the job number and name from Grid Engine's `qsub` output.
 
 
 class SgeLrms(batch.BatchSystem):
+
     """
     Job control on SGE clusters (possibly by connecting via SSH to a
     submit node).
@@ -357,7 +366,8 @@ class SgeLrms(batch.BatchSystem):
                  frontend, transport,
                  # these are specific to the SGE class
                  default_pe=None,
-                 # (Note that optional arguments to the `BatchSystem` class, e.g.:
+                 # (Note that optional arguments to the `BatchSystem` class,
+                 # e.g.:
                  #     keyfile=None, accounting_delay=15,
                  # are collected into `extra_args` and should not be explicitly
                  # spelled out in this signature.)
@@ -381,7 +391,8 @@ class SgeLrms(batch.BatchSystem):
 
     def _submit_command(self, app):
         sub_argv, app_argv = app.qsub_sge(self)
-        return (str.join(' ', sub_argv), str.join(' ', app_argv))
+        return (sh_quote_safe_cmdline(sub_argv),
+                sh_quote_unsafe_cmdline(app_argv))
 
     def _parse_submit_output(self, output):
         """Parse the ``qsub`` output for the local jobid."""
@@ -390,25 +401,29 @@ class SgeLrms(batch.BatchSystem):
     def _stat_command(self, job):
         return ("%s | egrep  '^ *%s'" % (self._qstat, job.lrms_jobid))
 
-    def _parse_stat_output(self, stdout):
-        job_status = stdout.split()[4]
-        log.debug("translating SGE's `qstat` code '%s' to gc3libs.Run.State",
-                  job_status)
-
-        jobstatus = dict()
-        if job_status in ['s', 'S', 'T'] or job_status.startswith('h'):
-            jobstatus['state'] = Run.State.STOPPED
-        elif 'qw' in job_status:
-            jobstatus['state'] = Run.State.SUBMITTED
-        elif 'r' in job_status or 'R' in job_status or 't' in job_status:
-            jobstatus['state'] = Run.State.RUNNING
-        elif job_status == 'E':  # error condition
-            jobstatus['state'] = Run.State.TERMINATING
+    def _parse_stat_output(self, stdout, stderr):
+        ge_status_code = stdout.split()[4]
+        log.debug(
+            "translating SGE's `qstat` code '%s' to gc3libs.Run.State",
+            ge_status_code)
+        if (ge_status_code in ['s', 'S', 'T']
+            or ge_status_code.startswith('h')):
+            state = Run.State.STOPPED
+        elif 'qw' in ge_status_code:
+            state = Run.State.SUBMITTED
+        elif ('r' in ge_status_code
+              or 'R' in ge_status_code
+              or 't' in ge_status_code):
+            state = Run.State.RUNNING
+        elif ge_status_code == 'E':  # error condition
+            state = Run.State.TERMINATING
         else:
             log.warning("unknown SGE job status '%s', returning `UNKNOWN`",
-                        job_status)
-            jobstatus['state'] = Run.State.UNKNOWN
-        return jobstatus
+                        ge_status_code)
+            state = Run.State.UNKNOWN
+        # to get the exit status information we'll have to parse
+        # `qacct` output so put ``None`` here
+        return self._stat_result(state, None)
 
     def _acct_command(self, job):
         return ("%s -j %s" % (self._qacct, job.lrms_jobid))
@@ -434,10 +449,11 @@ class SgeLrms(batch.BatchSystem):
         'jobname':       ('sge_jobname',         str),
         'qname':         ('sge_queue',           str),
         'qsub_time':     ('sge_submission_time', _parse_asctime),
-        'start_time':    ('sge_start_time',      _parse_asctime), }
+        'start_time':    ('sge_start_time', _parse_asctime),
+    }
 
-    def _parse_acct_output(self, stdout):
-        jobstatus = dict()
+    def _parse_acct_output(self, stdout, stderr):
+        acctinfo = {}
         for line in stdout.split('\n'):
             # skip empty and header lines
             line = line.strip()
@@ -451,16 +467,20 @@ class SgeLrms(batch.BatchSystem):
                 value = value.split()[0]
             try:
                 dest, conv = self._qacct_keyval_mapping[key]
-                jobstatus[dest] = conv(value)
+                acctinfo[dest] = conv(value)
             except KeyError:
                 # no conversion by default -- keep it a string
-                jobstatus['sge_' + key] = value
-            except (ValueError, TypeError), err:
+                acctinfo['sge_' + key] = value
+            except (ValueError, TypeError) as err:
                 log.error(
                     "Cannot parse value '%s' for qacct parameter '%s': %s: %s",
                     value, key, err.__class__.__name__, str(err))
-                jobstatus[dest] = None
-        return jobstatus
+                acctinfo[dest] = None
+        assert 'exitcode' in acctinfo, (
+            "Could not extract exit code from `tracejob` output")
+        acctinfo['termstatus'] = Run.shellexit_to_returncode(
+            acctinfo.pop('exitcode'))
+        return acctinfo
 
     def _cancel_command(self, jobid):
         return ("%s %s" % (self._qdel, jobid))
@@ -480,7 +500,7 @@ class SgeLrms(batch.BatchSystem):
                 raise gc3libs.exceptions.LRMSError(
                     "SGE backend failed executing '%s':"
                     "exit code: %d; stdout: '%s'; stderr: '%s'." %
-                    (_command, exit_code, stdout, stderr))
+                    (_command, exit_code, qstat_stdout, stderr))
 
             _command = ("%s -F -U %s" % (self._qstat, self._username))
             log.debug("Running `%s`...", _command)
@@ -491,7 +511,7 @@ class SgeLrms(batch.BatchSystem):
                 raise gc3libs.exceptions.LRMSError(
                     "SGE backend failed executing '%s':"
                     "exit code: %d; stdout: '%s'; stderr: '%s'." %
-                    (_command, exit_code, stdout, stderr))
+                    (_command, exit_code, qstat_F_stdout, stderr))
 
             (total_running, self.queued, self.user_run, self.user_queued) \
                 = count_jobs(qstat_stdout, self._username)
@@ -512,13 +532,13 @@ class SgeLrms(batch.BatchSystem):
                      )
             return self
 
-        except Exception, ex:
+        except Exception as ex:
             log.error("Error querying remote LRMS, see debug log for details.")
             log.debug("Error querying LRMS: %s: %s",
                       ex.__class__.__name__, str(ex))
             raise
 
-## main: run tests
+# main: run tests
 
 if "__main__" == __name__:
     import doctest
