@@ -3,7 +3,7 @@
 """
 Job control on SLURM clusters (possibly connecting to the front-end via SSH).
 """
-# Copyright (C) 2012-2013 GC3, University of Zurich. All rights reserved.
+# Copyright (C) 2012-2016, 2018, 2019  University of Zurich. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -16,59 +16,30 @@ Job control on SLURM clusters (possibly connecting to the front-end via SSH).
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
+from __future__ import absolute_import, print_function, unicode_literals
+from builtins import str
 __docformat__ = 'reStructuredText'
-__version__ = '2.2.3 version (SVN $Revision$)'
 
 
 import datetime
 import os
-import posixpath
-import random
 import re
-import sys
-import tempfile
 import time
 
-from gc3libs.compat._collections import defaultdict
-
 from gc3libs import log, Run
 from gc3libs.backends import LRMS
 import gc3libs.backends.batch as batch
 import gc3libs.exceptions
 from gc3libs.quantity import Memory, bytes, kB, MB, GB
-from gc3libs.quantity import Duration, seconds, minutes, hours
-import gc3libs.backends.transport as transport
-import gc3libs.utils as utils  # first, to_bytes
-from gc3libs.utils import same_docstring_as
-
-
-# environmental variables:
-#        SLURM_TIME_FORMAT Specify the format used to report time
-#        stamps. A value of standard, the default value, generates
-#        output in the form "year-month-dateThour:minute:second".
-
-# stat cmd: squeue --noheader --format='%i^%T^%r^%R'  -j jobid1,jobid2,...
-#   %i: job id
-#
-#   %T: Job state, extended form: PENDING, RUNNING, SUSPENDED,
-#       CANCELLED, COMPLETING, COMPLETED, CONFIGURING, FAILED, TIMEOUT,
-#       PREEMPTED, and NODE_FAIL.
-#
-#   %R: For pending jobs: the reason a job is waiting for execution is
-#       printed within parenthesis. For terminated jobs with failure: an
-#       explanation as to why the job failed is printed within
-#       parenthesis.  For all other job states: the list of allocate
-#       nodes.
-#   %r: reason a job is in its current state
+from gc3libs.quantity import Duration, seconds, minutes, hours, days
+from gc3libs.utils import (same_docstring_as, sh_quote_safe_cmdline,
+                           sh_quote_unsafe_cmdline)
 
 
 ## data for parsing SLURM commands output
 
-# regexps for extracting relevant strings
-
 # `sbatch` examples:
 #
 # $ sbatch -N4 myscript
@@ -82,14 +53,7 @@ from gc3libs.utils import same_docstring_as
 #
 _sbatch_jobid_re = re.compile(
     r'(sbatch:\s*)?(Granted job allocation|Submitted batch job)'
-    ' (?P<jobid>\d+)')
-
-# `squeue` examples:
-#
-#    $ squeue --noheader --format='%i|%T|%r|%R' -j 2,3
-#    2|PENDING|Resources|(Resources)
-#    3|PENDING|Resources|(Resources)
-#
+    r' (?P<jobid>\d+)')
 
 
 ## code
@@ -132,6 +96,7 @@ def count_jobs(squeue_output, whoami):
 
 
 class SlurmLrms(batch.BatchSystem):
+
     """
     Job control on SLURM clusters (possibly by connecting via SSH to a
     submit node).
@@ -146,11 +111,11 @@ class SlurmLrms(batch.BatchSystem):
                  auth,  # ignored if `transport` is 'local'
                  # these are inherited from `BatchSystem`
                  frontend, transport,
-                 # these are specific to this backend
-                 # (Note that optional arguments to the `BatchSystem` class, e.g.:
-                 #     keyfile=None, accounting_delay=15,
-                 # are collected into `extra_args` and should not be explicitly
-                 # spelled out in this signature.)
+                 # these are specific to this backend (Note that
+                 # optional arguments to the `BatchSystem` class,
+                 # e.g.: keyfile=None, accounting_delay=15, are
+                 # collected into `extra_args` and should not be
+                 # explicitly spelled out in this signature.)
                  **extra_args):
 
         # init base class
@@ -163,6 +128,7 @@ class SlurmLrms(batch.BatchSystem):
 
         # backend-specific setup
         self.sbatch = self._get_command_argv('sbatch')
+        self.srun = self._get_command_argv('srun')
 
         # SLURM commands
         self._scancel = self._get_command('scancel')
@@ -172,119 +138,153 @@ class SlurmLrms(batch.BatchSystem):
     def _parse_submit_output(self, output):
         return self.get_jobid_from_submit_output(output, _sbatch_jobid_re)
 
-    # submit cmd: sbatch --job-name="jobname" --mem-per-cpu="MBs" --input="filename" --output="filename" --no-requeue -n "number of slots" --cpus-per-task=1 --time="minutes" script.sh
+    # submit cmd: sbatch --job-name="jobname" --mem-per-cpu="MBs" --input="filename" --output="filename" --no-requeue -n "number of slots" --cpus-per-task=1 --time="minutes" script.sh  # noqa
     #
-    # You can only submit scripts with `sbatch`, attempts to directly run a command fail with this error message:
+    # You can only submit scripts with `sbatch`, attempts to directly
+    # run a command fail with this error message:
     #
     #    $ sbatch sleep 180
     #    sbatch: error: This does not look like a batch script.  The first
-    #    sbatch: error: line must start with #! followed by the path to an interpreter.
-    #    sbatch: error: For instance: #!/bin/sh
+    # sbatch: error: line must start with #! followed by the path to an interpreter.  # noqa
+    # sbatch: error: For instance: #!/bin/sh
     #
-    # If the server is not running or unreachable, the following error is displayed:
+    # If the server is not running or unreachable, the following error
+    # is displayed:
     #
     #    $ sbatch -n 1 --no-requeue /tmp/sleep.sh
-    #    sbatch: error: Batch job submission failed: Unable to contact slurm controller (connect failure)
+    # sbatch: error: Batch job submission failed: Unable to contact slurm controller (connect failure)  # noqa
     #
-    # Note: SLURM has a flexible way of assigning CPUs/cores/etc. to a job; here we treat all execution slots as equal and map 1 task to 1 core/thread.
+    # Note: SLURM has a flexible way of assigning CPUs/cores/etc. to a
+    # job; here we treat all execution slots as equal and map 1 task
+    # to 1 core/thread.
     #
-    # Acceptable time formats include "minutes",  "minutes:seconds", "hours:minutes:seconds", "days-hours", "days-hours:minutes" and "days-hours:minutes:seconds".
+    # Acceptable time formats include "minutes", "minutes:seconds",
+    # "hours:minutes:seconds", "days-hours", "days-hours:minutes" and
+    # "days-hours:minutes:seconds".
     #
     def _submit_command(self, app):
         sbatch_argv, app_argv = app.sbatch(self)
-        return (str.join(' ', sbatch_argv), str.join(' ', app_argv))
+        return (sh_quote_safe_cmdline(sbatch_argv),
+                sh_quote_unsafe_cmdline(app_argv))
 
-    # stat cmd: squeue --noheader --format='%i^%T^%u^%U^%r^%R'  -j jobid1,jobid2,...
+    # stat cmd: squeue --noheader --format='%i^%T^%u^%U^%r^%R'  -j jobid1,jobid2,...  # noqa
     #   %i: job id
-    #   %T: Job  state,  extended  form: PENDING, RUNNING, SUSPENDED, CANCELLED, COMPLETING, COMPLETED, CONFIGURING, FAILED, TIMEOUT, PREEMPTED, and NODE_FAIL.
-    #   %R: For pending jobs: the reason a job is waiting for execution is printed within parenthesis. For terminated jobs with failure: an  explanation  as to why the job failed is printed within parenthesis.  For all other job states: the list of allocate nodes.
+    #   %T: Job state, extended form: PENDING, RUNNING, SUSPENDED,
+    #       CANCELLED, COMPLETING, COMPLETED, CONFIGURING, FAILED,
+    #       TIMEOUT, PREEMPTED, and NODE_FAIL.
+    #   %R: For pending jobs: the reason a job is waiting for
+    #       execution is printed within parenthesis. For terminated
+    #       jobs with failure: an explanation as to why the job failed
+    #       is printed within parenthesis.  For all other job states:
+    #       the list of allocate nodes.
     #   %r: reason a job is in its current state
     #   %u: username of the submitting user
     #   %U: numeric UID of the submitting user
     #
     def _stat_command(self, job):
-        return "%s --noheader -o %%i^%%T^%%r -j %s" % \
-            (self._squeue, job.lrms_jobid)
+        return ("{squeue} --noheader -o GC3Pie^%%i^%%T^%%r -j {jobid}"
+                .format(squeue=self._squeue, jobid=job.lrms_jobid))
 
-    def _parse_stat_output(self, stdout):
+    def _parse_stat_output(self, stdout, stderr):
         """
-        Receive the output of ``squeue --noheader -o %i:%T:%r and parse it.
+        Parse output of ``squeue --noheader -o %i:%T:%r``.
         """
-        jobstatus = dict()
-        if stdout.strip() == '':
-            # if stdout is empty and `squeue -j` exitcode is 0, then
-            # the job has recently completed;
-            #
-            # if the job has been removed from the controllers'
-            # memory, then `squeue -j` exits with code 1
-            jobstatus['state'] = Run.State.TERMINATING
+        state = Run.State.UNKNOWN
+        for line in stdout.split('\n'):
+            line = line.strip()
+            # sites might wrap basic SLURM commands like `squeue` or
+            # `sacct` to provide additional information to users; we
+            # need to tell the actual SLURM output from the sites' own
+            # info; fortunately, SLURM's `--format` option allows
+            # arbitrary string prefixes which we can leverage to tag
+            # the interesting output lines.
+            if line.startswith('GC3Pie^'):
+                # parse stdout
+                _, job_id, job_state_code, reason = stdout.split('^')
+                log.debug(
+                    "translating SLURM state `%s` to gc3libs.Run.State",
+                    job_state_code)
+                if job_state_code in ['PENDING', 'CONFIGURING']:
+                    # XXX: see comments in `count_jobs` for a discussion
+                    # of whether 'CONFIGURING' should be grouped with
+                    # 'RUNNING' or not; here it's likely the correct
+                    # choice to group it with 'PENDING' as the
+                    # "configuring" phase may last a few minutes during
+                    # which the job is not yet really running.
+                    state = Run.State.SUBMITTED
+                elif job_state_code in ['RUNNING', 'COMPLETING']:
+                    state = Run.State.RUNNING
+                elif job_state_code in ['SUSPENDED']:
+                    state = Run.State.STOPPED
+                elif job_state_code in ['COMPLETED', 'CANCELLED', 'FAILED',
+                                        'NODE_FAIL', 'PREEMPTED', 'TIMEOUT']:
+                    state = Run.State.TERMINATING
+                else:
+                    state = Run.State.UNKNOWN
+                break
         else:
-            # parse stdout
-            jobid, state, reason = stdout.split('^')
-            log.debug("translating SLURM's state '%s' to gc3libs.Run.State",
-                      state)
-            if state in ['PENDING', 'CONFIGURING']:
-                # XXX: see above for a discussion of whether 'CONFIGURING'
-                # should be grouped with 'RUNNING' or not; here it's
-                # likely the correct choice to group it with 'PENDING' as
-                # the "configuring" phase may last a few minutes during
-                # which the job is not yet really running.
-                jobstatus['state'] = Run.State.SUBMITTED
-            elif state in ['RUNNING', 'COMPLETING']:
-                jobstatus['state'] = Run.State.RUNNING
-            elif state in ['SUSPENDED']:
-                jobstatus['state'] = Run.State.STOPPED
-            elif state in ['COMPLETED', 'CANCELLED', 'FAILED',
-                           'NODE_FAIL', 'PREEMPTED', 'TIMEOUT']:
-                jobstatus['state'] = Run.State.TERMINATING
-            else:
-                jobstatus['state'] = Run.State.UNKNOWN
-        return jobstatus
+            # No `GC3pie:` line found in output:
+            #
+            # * If stdout is empty and `squeue -j` exitcode is 0, then
+            # the job has recently completed (but we still need to
+            # call `sacct` to reap the termination status).
+            #
+            # * If the job has been removed from the controllers'
+            # memory, then `squeue -j` exits with code 1.
+            state = Run.State.TERMINATING
+        return self._stat_result(state, None)  # no term status info
 
-    # acct cmd: sacct --noheader --parsable --format jobid,ncpus,cputimeraw,elapsed,submit,eligible,reserved,start,end,exitcode,maxrss,maxvmsize,totalcpu -j JOBID
+    # acct cmd: sacct --noheader --parsable --format jobid,ncpus,cputimeraw,elapsed,submit,eligible,reserved,start,end,exitcode,maxrss,maxvmsize,totalcpu -j JOBID  # noqa
     #
     # where:
     #   * jobid:      Job ID
     #   * alloccpus:  Count of allocated processors
     #   * cputimeraw: CPU time in seconds
     #   * elapsed:    Job duration, as [DD-][hh:]mm:ss
-    #   * submit:     Time the job was submitted, as MM/DD-hh:mm:ss (timestamp in UTC) or ISO8601 format depending on compilation option
+    #   * submit:     Time the job was submitted, as MM/DD-hh:mm:ss
+    #                 (timestamp in UTC) or ISO8601 format depending on
+    #                 compilation option
     #   * eligible:   When the job became eligible to run
     #   * reserved:   Difference between `start` and `eligible`
-    #   * start:      Job start time, as MM/DD-hh:mm:ss (timestamp in UTC) or ISO8601 format depending on compilation option
-    #   * end:        Termination time of the job, as MM/DD-hh:mm:ss (timestamp in UTC) or ISO8601 format depending on compilation option
+    #   * start:      Job start time, as MM/DD-hh:mm:ss (timestamp in UTC) or
+    #                 ISO8601 format depending on compilation option
+    #   * end:        Termination time of the job, as MM/DD-hh:mm:ss
+    #                 (timestamp in UTC) or ISO8601 format depending on
+    #                 compilation option
     #   * exitcode:   exit code, ':', killing signal number
     #   * maxrss:     The maximum RSS across all tasks
     #   * maxvmsize:  The maximum virtual memory used across all tasks
-    #   * totalcpu:   Total CPU time used by the job (does not include child processes, if any)
+    #   * totalcpu: Total CPU time used by the job (does not include
+    #               child processes, if any)
     #
     # Examples:
     #
-    #    $ sudo sacct --noheader --parsable --format jobid,ncpus,cputimeraw,elapsed,submit,eligible,reserved,start,end,exitcode,maxrss,maxvmsize,totalcpu -j 14
-    #    14|1|10|00:00:10|2012-09-23T23:38:41|2012-09-23T23:38:41|49710-06:28:06|2012-09-23T23:38:31|2012-09-23T23:38:41|0:0|||00:00:00|
+    # $ sudo sacct --noheader --parsable --format jobid,ncpus,cputimeraw,elapsed,submit,eligible,reserved,start,end,exitcode,maxrss,maxvmsize,totalcpu -j 14  # noqa
+    # 14|1|10|00:00:10|2012-09-23T23:38:41|2012-09-23T23:38:41|49710-06:28:06|2012-09-23T23:38:31|2012-09-23T23:38:41|0:0|||00:00:00|  # noqa
     #
-    #    $ sudo sacct --noheader --parsable --format jobid,ncpus,cputimeraw,elapsed,submit,start,end,exitcode,maxrss,maxvmsize,totalcpu -j 19
-    #    19|1|66|00:01:06|2012-09-24T10:48:34|2012-09-24T10:47:28|2012-09-24T10:48:34|0:0|||00:00:00|
-    #    19.0|1|65|00:01:05|2012-09-24T10:47:29|2012-09-24T10:47:29|2012-09-24T10:48:34|0:0|0|0|00:00:00|
+    # $ sudo sacct --noheader --parsable --format jobid,ncpus,cputimeraw,elapsed,submit,start,end,exitcode,maxrss,maxvmsize,totalcpu -j 19  # noqa
+    # 19|1|66|00:01:06|2012-09-24T10:48:34|2012-09-24T10:47:28|2012-09-24T10:48:34|0:0|||00:00:00|      # noqa
+    # 19.0|1|65|00:01:05|2012-09-24T10:47:29|2012-09-24T10:47:29|2012-09-24T10:48:34|0:0|0|0|00:00:00|  # noqa
     #
-    #    $ env SLURM_TIME_FORMAT=standard sacct -S 0901 --noheader --parsable --format jobid,ncpus,cputimeraw,elapsed,submit,eligible,reserved,start,end,exitcode,maxrss,maxvmsize,totalcpu
-    #    449002|32|128|00:00:04|2012-09-04T11:09:26|2012-09-04T11:09:38|2012-09-04T11:09:42|0:0|||00:00:00|
-    #    449018|64|1472|00:00:23|2012-09-04T11:18:06|2012-09-04T11:18:24|2012-09-04T11:18:47|0:0|||00:01.452|
-    #    449018.batch|1|23|00:00:23|2012-09-04T11:18:24|2012-09-04T11:18:24|2012-09-04T11:18:47|0:0|7884K|49184K|00:01.452|
-    #    449057|3200|659200|00:03:26|2012-09-04T11:19:45|2012-09-04T11:19:57|2012-09-04T11:23:23|0:0|||00:01.620|
-    #    449057.batch|1|206|00:03:26|2012-09-04T11:19:57|2012-09-04T11:19:57|2012-09-04T11:23:23|0:0|7896K|49184K|00:01.620|
+    # $ env SLURM_TIME_FORMAT=standard sacct -S 0901 --noheader --parsable --format jobid,ncpus,cputimeraw,elapsed,submit,eligible,reserved,start,end,exitcode,maxrss,maxvmsize,totalcpu  # noqa
+    # 449002|32|128|00:00:04|2012-09-04T11:09:26|2012-09-04T11:09:38|2012-09-04T11:09:42|0:0|||00:00:00|                  # noqa
+    # 449018|64|1472|00:00:23|2012-09-04T11:18:06|2012-09-04T11:18:24|2012-09-04T11:18:47|0:0|||00:01.452|                # noqa
+    # 449018.batch|1|23|00:00:23|2012-09-04T11:18:24|2012-09-04T11:18:24|2012-09-04T11:18:47|0:0|7884K|49184K|00:01.452|  # noqa
+    # 449057|3200|659200|00:03:26|2012-09-04T11:19:45|2012-09-04T11:19:57|2012-09-04T11:23:23|0:0|||00:01.620|            # noqa
+    # 449057.batch|1|206|00:03:26|2012-09-04T11:19:57|2012-09-04T11:19:57|2012-09-04T11:23:23|0:0|7896K|49184K|00:01.620| # noqa
     #
-    # Warning: the `SLURM_TIME_FORMAT` environment variable influences how the times are reported,
-    # so it should always be set to `standard` to get ISO8601 reporting.  See below for an example
+    # Warning: the `SLURM_TIME_FORMAT` environment variable influences
+    # how the times are reported, so it should always be set to
+    # `standard` to get ISO8601 reporting.  See below for an example
     # of non-standard (SLURM_TIME_FORMAT=relative) report:
     #
-    #    $ sacct --noheader --parsable --format jobid,ncpus,cputimeraw,elapsed,submit,eligible,reserved,start,end,exitcode,maxrss,maxvmsize,totalcpu -S 0901
-    #    449002|32|128|00:00:04|4 Sep 11:09|4 Sep 11:09|00:00:12|4 Sep 11:09|4 Sep 11:09|0:0|||00:00:00|
-    #    449018|64|1472|00:00:23|4 Sep 11:18|4 Sep 11:18|00:00:18|4 Sep 11:18|4 Sep 11:18|0:0|||00:01.452|
-    #    449018.batch|1|23|00:00:23|4 Sep 11:18|4 Sep 11:18|00:00:-02|4 Sep 11:18|4 Sep 11:18|0:0|7884K|49184K|00:01.452|
-    #    449057|3200|659200|00:03:26|4 Sep 11:19|4 Sep 11:19|00:00:12|4 Sep 11:19|4 Sep 11:23|0:0|||00:01.620|
-    #    449057.batch|1|206|00:03:26|4 Sep 11:19|4 Sep 11:19|00:00:-02|4 Sep 11:19|4 Sep 11:23|0:0|7896K|49184K|00:01.620|
-    #    449217|3200|0|00:00:00|4 Sep 11:42|4 Sep 11:42|00:00:05|4 Sep 11:43|4 Sep 11:43|0:0|||00:00.720|
+    # $ sacct --noheader --parsable --format jobid,ncpus,cputimeraw,elapsed,submit,eligible,reserved,start,end,exitcode,maxrss,maxvmsize,totalcpu -S 0901  # noqa
+    # 449002|32|128|00:00:04|4 Sep 11:09|4 Sep 11:09|00:00:12|4 Sep 11:09|4 Sep 11:09|0:0|||00:00:00|                    # noqa
+    # 449018|64|1472|00:00:23|4 Sep 11:18|4 Sep 11:18|00:00:18|4 Sep 11:18|4 Sep 11:18|0:0|||00:01.452|                  # noqa
+    # 449018.batch|1|23|00:00:23|4 Sep 11:18|4 Sep 11:18|00:00:-02|4 Sep 11:18|4 Sep 11:18|0:0|7884K|49184K|00:01.452|   # noqa
+    # 449057|3200|659200|00:03:26|4 Sep 11:19|4 Sep 11:19|00:00:12|4 Sep 11:19|4 Sep 11:23|0:0|||00:01.620|              # noqa
+    # 449057.batch|1|206|00:03:26|4 Sep 11:19|4 Sep 11:19|00:00:-02|4 Sep 11:19|4 Sep 11:23|0:0|7896K|49184K|00:01.620|  # noqa
+    # 449217|3200|0|00:00:00|4 Sep 11:42|4 Sep 11:42|00:00:05|4 Sep 11:43|4 Sep 11:43|0:0|||00:00.720|                   # noqa
     #
     # If SLURM accounting is disabled (default), then `sacct` outputs
     # an error message:
@@ -293,36 +293,69 @@ class SlurmLrms(batch.BatchSystem):
     #    SLURM accounting storage is disabled
     #
     def _acct_command(self, job):
-        return ('env SLURM_TIME_FORMAT=standard %s --noheader --parsable' \
-                ' --format jobid,exitcode,ncpus,elapsed,totalcpu,' \
-                'submit,start,end,maxrss,maxvmsize -j %s' % \
+        return ('env SLURM_TIME_FORMAT=standard %s --noheader --parsable'
+                ' --format jobid,exitcode,state,ncpus,elapsed,totalcpu,'
+                'submit,start,end,maxrss,maxvmsize -j %s' %
                 (self._sacct, job.lrms_jobid))
 
-    def _parse_acct_output(self, stdout):
-        acct = dict(
-            exitcode=0,
-            cores=0,
-            duration=Duration(0, unit=seconds),
-            used_cpu_time=Duration(0, unit=seconds),
-            max_used_memory=Memory(0, unit=bytes))
+    def _parse_acct_output(self, stdout, stderr):
+        acct = {
+            'cores':            0,
+            'duration':         Duration(0, unit=seconds),
+            'used_cpu_time':    Duration(0, unit=seconds),
+            'max_used_memory':  Memory(0, unit=bytes)
+        }
+        exitcode = None
+        signal = None
         for line in stdout.split('\n'):
             line = line.strip()
             if line == '':
                 continue
             # because of the trailing `|` we have an extra empty field
-            jobid, exit, ncpus, elapsed, totalcpu, submit,\
+            jobid, exit, state, ncpus, elapsed, totalcpu, submit,\
                 start, end, maxrss, maxvmsize, _ = line.split('|')
+
+            # In some case the state can contain a specification,
+            # e.g. "CANCELLED by 1000"
+            state = state.split()[0]
+
             # SLURM job IDs have the form `jobID[.step]`: only the
             # lines with the `step` part carry resource usage records,
             # whereas the total `jobID` line carries the exit codes
             # and overall duration/timing information.
             if '.' not in jobid:
+                if state not in [
+                        'BOOT_FAIL',
+                        'CANCELLED',
+                        'COMPLETED',
+                        'FAILED',
+                        'NODE_FAIL',
+                        'PREEMPTED',
+                        'TIMEOUT',
+                ]:
+                    raise gc3libs.exceptions.UnexpectedJobState(
+                        "Unexpected SLURM job state '{state}'"
+                        " encountered in parsing `sacct` output"
+                        .format(state=state)
+                    )
                 # master job record
                 acct['duration'] = SlurmLrms._parse_duration(elapsed)
                 acct['used_cpu_time'] = SlurmLrms._parse_duration(totalcpu)
-                # compute POSIX exit status
-                exitcode, signal = exit.split(':')
-                acct['exitcode'] = (int(exitcode) << 8) + (int(signal) & 0x7f)
+                if state in ['CANCELLED', 'TIMEOUT']:
+                    # In this case, the exit code of the master job is
+                    # `0:0` or `0:1`, but we want to keep track of the
+                    # fact that the job was killed by the system (or
+                    # the user).
+                    exitcode = os.EX_TEMPFAIL
+                    signal = int(Run.Signals.RemoteKill)
+                elif state == 'NODE_FAIL':
+                    exitcode = os.EX_TEMPFAIL
+                    signal = int(Run.Signals.RemoteError)
+                else:
+                    # compute POSIX exit status
+                    exitcode_, signal_ = exit.split(':')
+                    exitcode = int(exitcode_)
+                    signal = int(signal_)
                 # XXX: the master job record seems to report the
                 # *requested* slots, whereas the step records report
                 # the actual usage.  In our case these should be the
@@ -343,24 +376,26 @@ class SlurmLrms(batch.BatchSystem):
                 start = SlurmLrms._parse_timestamp(start)
                 end = SlurmLrms._parse_timestamp(end)
                 acct['slurm_submission_time'] = min(submit, start)
-                acct['slurm_start_time'] = end  # will be set when
-                                                # looping on tasks,
-                                                # see below
+                acct['slurm_start_time'] = end  # actually computed below
                 acct['slurm_completion_time'] = max(submit, start, end)
             else:
                 # common resource usage records (see Issue 78)
                 vmem = SlurmLrms._parse_memspec(maxvmsize)
-                acct['max_used_memory'] = max(vmem, acct['max_used_memory'])
+                if vmem is not None:
+                    acct['max_used_memory'] = max(vmem, acct['max_used_memory'])
                 # SLURM-specific resource usage records
                 mem = SlurmLrms._parse_memspec(maxrss)
-                acct['slurm_max_used_ram'] = max(
-                    mem, acct['slurm_max_used_ram'])
+                if mem is not None:
+                    acct['slurm_max_used_ram'] = max(mem, acct['slurm_max_used_ram'])
                 # XXX: see above for timestamps
                 submit = SlurmLrms._parse_timestamp(submit)
                 start = SlurmLrms._parse_timestamp(start)
                 acct['slurm_submission_time'] = min(
                     submit, acct['slurm_submission_time'])
                 acct['slurm_start_time'] = min(start, acct['slurm_start_time'])
+        # must compute termination status since it's not provided by `squeue`
+        if signal is not None and exitcode is not None:
+            acct['termstatus'] = (signal & 0x7f) + ((exitcode & 0xff) << 8)
         return acct
 
     @staticmethod
@@ -391,6 +426,10 @@ class SlurmLrms(batch.BatchSystem):
 
     @staticmethod
     def _parse_memspec(m):
+        # upon NODE_FAIL (and possibly some other occasions), SLURM does not
+        # report usage information -- return ``None`` in this case
+        if not m:
+            return None
         unit = m[-1]
         if unit == 'G':
             return Memory(int(round(float(m[:-1]))), unit=GB)
@@ -410,11 +449,9 @@ class SlurmLrms(batch.BatchSystem):
         The 'standard' format for SLURM timestamps is ISO8601;
         raise an error if any other format is detected.
         """
-        # XXX: datetime.strptime() only available starting Py 2.5
         try:
-            return datetime.datetime(
-                *(time.strptime(ts, SlurmLrms._TIMEFMT_ISO8601)[0:6]))
-        except ValueError, err:
+            return datetime.datetime.strptime(ts, SlurmLrms._TIMEFMT_ISO8601)
+        except ValueError as err:
             gc3libs.log.error(
                 "Could not parse '%s' as an SLURM 'standard' (ISO8601)"
                 " timestamp: %s: %s Please set the environment variable"
@@ -484,7 +521,7 @@ class SlurmLrms(batch.BatchSystem):
                      )
             return self
 
-        except Exception, ex:
+        except Exception as ex:
             # self.transport.close()
             log.error("Error querying remote LRMS, see debug log for details.")
             log.debug("Error querying LRMS: %s: %s",
@@ -492,7 +529,7 @@ class SlurmLrms(batch.BatchSystem):
             raise
 
 
-## main: run tests
+# main: run tests
 
 if "__main__" == __name__:
     import doctest
