@@ -3,7 +3,7 @@
 """
 Unit tests for the `gc3libs.backends.shellcmd` module.
 """
-# Copyright (C) 2011-2013 GC3, University of Zurich. All rights reserved.
+# Copyright (C) 2011-2015, 2018  University of Zurich. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -16,27 +16,33 @@ Unit tests for the `gc3libs.backends.shellcmd` module.
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
+from __future__ import absolute_import, print_function, unicode_literals
+from future import standard_library
+standard_library.install_aliases()
+from builtins import object
 __docformat__ = 'reStructuredText'
-__version__ = '$Revision$'
 
 
-import errno
+from io import StringIO
 import os
 import shutil
-import sys
 import tempfile
 import time
 
-from nose.tools import raises, assert_equal, assert_not_equal
+import pytest
 
 import gc3libs
-from gc3libs.authentication import Auth
+from gc3libs.backends.shellcmd import ShellcmdLrms
 import gc3libs.config
 import gc3libs.core
-from gc3libs.quantity import Memory
+from gc3libs.quantity import Duration, Memory, s, kB
+from gc3libs.testing.helpers import (
+    SuccessfulApp,
+    temporary_core,
+    temporary_directory,
+)
 
 
 class TestBackendShellcmd(object):
@@ -60,6 +66,7 @@ resourcedir=%s
 type=none
 """
 
+    @pytest.fixture(autouse=True)
     def setUp(self):
         (fd, cfgfile) = tempfile.mkstemp()
         f = os.fdopen(fd, 'w+')
@@ -77,10 +84,8 @@ type=none
         # Update resource status
         self.backend.get_resource_status()
 
-    def cleanup_file(self, fname):
-        self.files_to_remove.append(fname)
+        yield
 
-    def tearDown(self):
         for fname in self.files_to_remove:
             if os.path.isdir(fname):
                 shutil.rmtree(fname)
@@ -97,11 +102,24 @@ type=none
             except:
                 pass
 
+    def cleanup_file(self, fname):
+        self.files_to_remove.append(fname)
+
+    def run_until_terminating(self, app, max_wait=10, polling_interval=0.1):
+        """
+        Wait until the given `app` task is done,
+        but timeout and raise an error if it takes too much time.
+        """
+        waited = 0
+        while (app.execution.state != gc3libs.Run.State.TERMINATING
+                and waited < max_wait):
+            time.sleep(polling_interval)
+            waited += polling_interval
+            self.core.update_job_state(app)
+
+
     def test_submission_ok(self):
-        """
-        Test a successful submission cycle and the backends' resource
-        book-keeping.
-        """
+        """Test a successful submission cycle and the backends' resource book-keeping"""
         tmpdir = tempfile.mkdtemp(prefix=__name__, suffix='.d')
 
         app = gc3libs.Application(
@@ -120,26 +138,17 @@ type=none
 
         # there's no SUBMITTED state here: jobs go immediately into
         # RUNNING state
-        assert_equal(app.execution.state, gc3libs.Run.State.SUBMITTED)
-        assert_equal(self.backend.free_slots,  122)
-        assert_equal(self.backend.user_queued, 0)
-        assert_equal(self.backend.user_run,    1)
+        assert app.execution.state == gc3libs.Run.State.SUBMITTED
+        assert self.backend.free_slots == 122
+        assert self.backend.user_queued == 0
+        assert self.backend.user_run == 1
 
-        # wait until the test job is done, but timeout and raise an error
-        # if it takes too much time...
-        MAX_WAIT = 10  # seconds
-        WAIT = 0.1  # seconds
-        waited = 0
-        while app.execution.state != gc3libs.Run.State.TERMINATING \
-                and waited < MAX_WAIT:
-            time.sleep(WAIT)
-            waited += WAIT
-            self.core.update_job_state(app)
+        self.run_until_terminating(app)
         try:
-            assert_equal(app.execution.state, gc3libs.Run.State.TERMINATING)
-            assert_equal(self.backend.free_slots,  123)
-            assert_equal(self.backend.user_queued, 0)
-            assert_equal(self.backend.user_run,    0)
+            assert app.execution.state == gc3libs.Run.State.TERMINATING
+            assert self.backend.free_slots == 123
+            assert self.backend.user_queued == 0
+            assert self.backend.user_run == 0
         except:
             self.core.fetch_output(app)
             self.core.free(app)
@@ -147,18 +156,17 @@ type=none
 
         self.core.fetch_output(app)
         try:
-            assert_equal(app.execution.state, gc3libs.Run.State.TERMINATED)
-            assert_equal(self.backend.free_slots,  123)
-            assert_equal(self.backend.user_queued, 0)
-            assert_equal(self.backend.user_run,    0)
+            assert app.execution.state == gc3libs.Run.State.TERMINATED
+            assert self.backend.free_slots == 123
+            assert self.backend.user_queued == 0
+            assert self.backend.user_run == 0
         except:
             self.core.free(app)
             raise
 
     def test_check_app_after_reloading_session(self):
-        """Check if we are able to check the status of a job after the
-        script which started the job has died.
-        """
+        """Check that the job status is still available the end of the starter script"""
+
         tmpdir = tempfile.mkdtemp(prefix=__name__, suffix='.d')
         self.cleanup_file(tmpdir)
 
@@ -174,29 +182,16 @@ type=none
         self.apps_to_kill.append(app)
 
         self.cleanup_file(app.execution.lrms_execdir)
-        pid = app.execution.lrms_jobid
 
         # The wrapper process should die and write the final status
         # and the output to a file, so that `Core` will be able to
         # retrieve it.
-
-        # wait until the test job is done, but timeout and raise an error
-        # if it takes too much time...
-        MAX_WAIT = 10  # seconds
-        WAIT = 0.1  # seconds
-        waited = 0
-        while app.execution.state != gc3libs.Run.State.TERMINATING \
-                and waited < MAX_WAIT:
-            time.sleep(WAIT)
-            waited += WAIT
-            self.core.update_job_state(app)
-
-        assert_equal(app.execution.state, gc3libs.Run.State.TERMINATING)
-        assert_equal(app.execution.returncode, 0)
+        self.run_until_terminating(app)
+        assert app.execution.state == gc3libs.Run.State.TERMINATING
+        assert app.execution.returncode == 0
 
     def test_app_argument_with_spaces(self):
-        """Check that arguments with spaces are not splitted
-        """
+        """Check that arguments with spaces are not split"""
         tmpdir = tempfile.mkdtemp(prefix=__name__, suffix='.d')
         self.cleanup_file(tmpdir)
 
@@ -210,23 +205,14 @@ type=none
             requested_cores=1, )
         self.core.submit(app)
         self.apps_to_kill.append(app)
-
         self.cleanup_file(app.execution.lrms_execdir)
-        MAX_WAIT = 10  # seconds
-        WAIT = 0.1  # seconds
-        waited = 0
-        while app.execution.state != gc3libs.Run.State.TERMINATING \
-                and waited < MAX_WAIT:
-            time.sleep(WAIT)
-            waited += WAIT
-            self.core.update_job_state(app)
-        assert_equal(app.execution.state, gc3libs.Run.State.TERMINATING)
-        assert_not_equal(app.execution.returncode, 0)
 
-    # def test_time_cmd_args(self):
-    #     assert_equal(self.backend.time_cmd, '/usr/bin/time')
+        self.run_until_terminating(app)
+        assert app.execution.state == gc3libs.Run.State.TERMINATING
+        assert app.execution.returncode != 0
 
     def test_resource_usage(self):
+        """Check book-keeping of core and memory resources"""
         tmpdir = tempfile.mkdtemp(prefix=__name__, suffix='.d')
         self.cleanup_file(tmpdir)
 
@@ -244,22 +230,64 @@ type=none
 
         cores_after = self.backend.free_slots
         mem_after = self.backend.available_memory
-        assert_equal(cores_before, cores_after + 2)
-        assert_equal(mem_before, mem_after + app.requested_memory)
-        MAX_WAIT = 10  # seconds
-        WAIT = 0.1  # seconds
-        waited = 0
-        while app.execution.state != gc3libs.Run.State.TERMINATING \
-                and waited < MAX_WAIT:
-            time.sleep(WAIT)
-            waited += WAIT
-            self.core.update_job_state(app)
-        assert_equal(self.backend.free_slots, cores_before)
-        avail = self.backend.available_memory
-        assert_equal(self.backend.available_memory, mem_before)
+        assert cores_before == cores_after + 2
+        assert mem_before == mem_after + app.requested_memory
+        self.run_until_terminating(app)
+        assert self.backend.free_slots == cores_before
+        assert self.backend.available_memory == mem_before
+
+    def test_env_vars_definition(self):
+        """Check that `Application.environment` settings are correctly propagated"""
+        tmpdir = tempfile.mkdtemp(prefix=__name__, suffix='.d')
+        self.cleanup_file(tmpdir)
+
+        app = gc3libs.Application(
+            arguments=['/bin/echo', '$MSG'],
+            inputs=[],
+            outputs=[],
+            output_dir=tmpdir,
+            stdout='stdout.txt',
+            environment={'MSG':'OK'})
+        self.core.submit(app)
+        self.apps_to_kill.append(app)
+
+        self.run_until_terminating(app)
+
+        self.core.fetch_output(app)
+        stdout_file = os.path.join(app.output_dir, app.stdout)
+        assert os.path.exists(stdout_file)
+        assert os.path.isfile(stdout_file)
+        stdout_contents = open(stdout_file, 'r').read()
+        assert stdout_contents == 'OK\n'
+
+    def test_stdout_in_directory(self):
+        """Check that `Application.stdout` can include a full path"""
+        tmpdir = tempfile.mkdtemp(prefix=__name__, suffix='.d')
+        self.cleanup_file(tmpdir)
+
+        app = gc3libs.Application(
+            arguments=['/bin/echo', 'OK'],
+            inputs=[],
+            outputs=[],
+            output_dir=tmpdir,
+            stdout='logs/stdout.txt')
+        self.core.submit(app)
+        self.apps_to_kill.append(app)
+
+        self.run_until_terminating(app)
+
+        self.core.fetch_output(app)
+        stdout_dir = os.path.join(app.output_dir, 'logs')
+        stdout_file = os.path.join(stdout_dir, 'stdout.txt')
+        assert os.path.exists(stdout_dir)
+        assert os.path.isdir(stdout_dir)
+        assert os.path.exists(stdout_file)
+        assert os.path.isfile(stdout_file)
+        stdout_contents = open(stdout_file, 'r').read()
+        assert stdout_contents == 'OK\n'
 
-    @raises(gc3libs.exceptions.NoResources)
     def test_not_enough_cores_usage(self):
+        """Check that a `NoResources` exception is raised if more cores are requested than available"""
         tmpdir = tempfile.mkdtemp(prefix=__name__, suffix='.d')
         self.cleanup_file(tmpdir)
         bigapp = gc3libs.Application(
@@ -269,10 +297,11 @@ type=none
             output_dir=tmpdir,
             requested_cores=self.backend.free_slots + 1,
             requested_memory=10 * Memory.MiB, )
-        self.core.submit(bigapp)
+        with pytest.raises(gc3libs.exceptions.NoResources):
+            self.core.submit(bigapp)
 
-    @raises(gc3libs.exceptions.NoResources)
     def test_not_enough_memory_usage(self):
+        """Check that a `NoResources` exception is raised if more memory is requested than available"""
         tmpdir = tempfile.mkdtemp(prefix=__name__, suffix='.d')
         self.cleanup_file(tmpdir)
         bigapp = gc3libs.Application(
@@ -282,7 +311,8 @@ type=none
             output_dir=tmpdir,
             requested_cores=1,
             requested_memory=self.backend.total_memory + Memory.B, )
-        self.core.submit(bigapp)
+        with pytest.raises(gc3libs.exceptions.NoResources):
+            self.core.submit(bigapp)
 
 
 class TestBackendShellcmdCFG(object):
@@ -305,19 +335,21 @@ resourcedir=%s
 type=none
 """
 
+    @pytest.fixture(autouse=True)
     def setUp(self):
         self.files_to_remove = []
 
-    def cleanup_file(self, fname):
-        self.files_to_remove.append(fname)
-
-    def tearDown(self):
+        yield
         for fname in self.files_to_remove:
             if os.path.isdir(fname):
                 shutil.rmtree(fname)
             elif os.path.exists(fname):
                 os.remove(fname)
 
+    def cleanup_file(self, fname):
+        self.files_to_remove.append(fname)
+
+
     def test_override_cfg_flag(self):
         (fd, cfgfile) = tempfile.mkstemp()
         f = os.fdopen(fd, 'w+')
@@ -349,7 +381,7 @@ type=none
         self.backend = self.core.get_backend('localhost_test')
         # Update resource status
 
-        assert_equal(self.backend.max_cores, 1000)
+        assert self.backend.max_cores == 1000
 
     def test_resource_sharing_w_multiple_backends(self):
         tmpdir = tempfile.mkdtemp(prefix=__name__, suffix='.d')
@@ -381,16 +413,140 @@ type=none
 
         try:
             core1.submit(app)
-            assert_equal(backend1.free_slots, backend1.max_cores - app.requested_cores)
+            assert (backend1.free_slots ==
+                         backend1.max_cores - app.requested_cores)
 
-            assert_equal(backend2.free_slots, backend2.max_cores)
+            assert backend2.free_slots == backend2.max_cores
             backend2.get_resource_status()
-            assert_equal(backend2.free_slots, backend2.max_cores - app.requested_cores)
+            assert (backend2.free_slots ==
+                         backend2.max_cores - app.requested_cores)
         finally:
             core1.kill(app)
             core1.free(app)
 
 
+    def test_pid_list_is_string(self):
+        tmpdir = tempfile.mkdtemp(prefix=__name__, suffix='.d')
+        (fd, cfgfile) = tempfile.mkstemp()
+        f = os.fdopen(fd, 'w+')
+        f.write(TestBackendShellcmdCFG.CONF % ("False", cfgfile + '.d'))
+        f.close()
+        self.files_to_remove = [cfgfile, cfgfile + '.d']
+
+        self.cfg = gc3libs.config.Configuration()
+        self.cfg.merge_file(cfgfile)
+
+        self.core = gc3libs.core.Core(self.cfg)
+        self.backend = self.core.get_backend('localhost_test')
+        # Update resource status
+
+        app = gc3libs.Application(
+        arguments=['/bin/echo', 'Hello', 'World'],
+            inputs=[],
+            outputs=[],
+            output_dir=tmpdir,
+            requested_cores=1,
+            requested_memory=10 * Memory.MiB, )
+
+        try:
+            self.core.submit(app)
+            for pid in list(self.backend._job_infos.keys()):
+                assert isinstance(pid,str)
+        finally:
+            self.core.kill(app)
+            self.core.free(app)
+
+def test_shellcmd_backend_create_spooldir():
+    with temporary_directory() as tmpdir:
+        with temporary_core(spooldir=tmpdir) as core:
+            assert 'test' in core.resources
+            backend = core.get_backend('test')
+            app = SuccessfulApp()
+            core.submit(app)
+            assert os.path.isdir(backend.spooldir)
+
+
+def test_shellcmd_backend_parse_wrapper_output_successful():
+    resource_usage = StringIO("""
+WallTime=2.10s
+KernelTime=0.61s
+UserTime=0.67s
+CPUUsage=61%
+MaxResidentMemory=61604kB
+AverageResidentMemory=0kB
+AverageTotalMemory=0kB
+AverageUnsharedMemory=0kB
+AverageUnsharedStack=0kB
+AverageSharedMemory=0kB
+PageSize=4096B
+MajorPageFaults=0
+MinorPageFaults=101799
+Swaps=0
+ForcedSwitches=82
+WaitSwitches=487
+Inputs=0
+Outputs=400
+SocketReceived=0
+SocketSent=0
+Signals=0
+ReturnCode=0
+    """)
+    with temporary_core(type='shellcmd') as core:
+        assert 'test' in core.resources
+        backend = core.get_backend('test')
+        termstatus, acctinfo, valid = backend._parse_wrapper_output(resource_usage)
+        assert valid
+        assert acctinfo.duration == 2.10*s
+        assert acctinfo.shellcmd_kernel_time == 0.61*s
+        assert acctinfo.shellcmd_user_time == 0.67*s
+        assert acctinfo.max_used_memory == 61604*kB
+        assert acctinfo.shellcmd_average_total_memory == 0*kB
+        assert acctinfo.shellcmd_filesystem_inputs == 0
+        assert acctinfo.shellcmd_filesystem_outputs == 400
+        assert acctinfo.shellcmd_swapped == 0
+        assert termstatus == (0, 0)
+
+
+def test_shellcmd_backend_parse_wrapper_output_signalled():
+    resource_usage = StringIO("""Command terminated by signal 15
+WallTime=6.44s
+KernelTime=0.00s
+UserTime=0.00s
+CPUUsage=0%
+MaxResidentMemory=2004kB
+AverageResidentMemory=0kB
+AverageTotalMemory=0kB
+AverageUnsharedMemory=0kB
+AverageUnsharedStack=0kB
+AverageSharedMemory=0kB
+PageSize=4096B
+MajorPageFaults=0
+MinorPageFaults=73
+Swaps=0
+ForcedSwitches=0
+WaitSwitches=2
+Inputs=0
+Outputs=0
+SocketReceived=0
+SocketSent=0
+Signals=0
+ReturnCode=0
+    """)
+    with temporary_core(type='shellcmd') as core:
+        assert 'test' in core.resources
+        backend = core.get_backend('test')
+        termstatus, acctinfo, valid = backend._parse_wrapper_output(resource_usage)
+        assert valid
+        assert acctinfo.duration == 6.44*s
+        assert acctinfo.shellcmd_kernel_time == 0.*s
+        assert acctinfo.shellcmd_user_time == 0.*s
+        assert acctinfo.max_used_memory == 2004*kB
+        assert acctinfo.shellcmd_average_total_memory == 0*kB
+        assert acctinfo.shellcmd_filesystem_inputs == 0
+        assert acctinfo.shellcmd_filesystem_outputs == 0
+        assert acctinfo.shellcmd_swapped == 0
+        assert termstatus == (15, -1)
+
+
 if __name__ == "__main__":
-    import nose
-    nose.runmodule()
+    pytest.main(["-v", __file__])
