@@ -16,12 +16,11 @@ the packages in the `gc3libs.compat` namespace.
 """
 # This file is only present so that the `compat/` directory is
 # recognized as a Python package.
+from __future__ import absolute_import, print_function, unicode_literals
 __docformat__ = 'reStructuredText'
-__version__ = '$Revision$'
 
 
-
-## main: run tests
+# main: run tests
 
 if "__main__" == __name__:
     import doctest
