@@ -1,4 +1,5 @@
 #! /usr/bin/env python
+
 """Exceptions specific to the `gc3libs` package.
 
 In addition to the exceptions listed here, `gc3libs`:mod: functions
@@ -19,7 +20,8 @@ in core Python, namely:
   this indicates a bug in the software.
 
 """
-# Copyright (C) 2009-2014 GC3, University of Zurich. All rights reserved.
+
+# Copyright (C) 2009-2019  University of Zurich. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -32,18 +34,22 @@ in core Python, namely:
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-#
-__docformat__ = 'reStructuredText'
-__version__ = '2.2.3 version (SVN $Revision: 4001 $)'
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+from __future__ import absolute_import, print_function, unicode_literals
+from builtins import str
+__docformat__ = 'reStructuredText'
+
+
+from warnings import warn
 
 import gc3libs
 
+
 ## base error classes
 
 class Error(Exception):
+
     """
     Base class for all error-level exceptions in GC3Pie.
 
@@ -56,17 +62,19 @@ class Error(Exception):
     cannot continue and should immediately stop, should use the
     `FatalError`:class: base class.
     """
+
     def __init__(self, msg, do_log=False):
         if do_log:
             gc3libs.log.error(msg)
         Exception.__init__(self, msg)
 
 
-## mark errors as "Recoverable" (meaning that a retry a ta later time
-## could succeed), or "Unrecoverable" (meaning there's no point in
-## retrying).
+# mark errors as "Recoverable" (meaning that a retry a ta later time
+# could succeed), or "Unrecoverable" (meaning there's no point in
+# retrying).
 
 class RecoverableError(Error):
+
     """
     Used to mark transient errors: retrying the same action at a later
     time could succeed.
@@ -78,6 +86,7 @@ class RecoverableError(Error):
 
 
 class UnrecoverableError(Error):
+
     """
     Used to mark permanent errors: there's no point in retrying the same
     action at a later time, because it will yield the same error again.
@@ -89,6 +98,7 @@ class UnrecoverableError(Error):
 
 
 class FatalError(UnrecoverableError):
+
     """
     A fatal error: execution cannot continue and program should report
     to user and then stop.
@@ -98,6 +108,7 @@ class FatalError(UnrecoverableError):
 
     This is the base class for all fatal exceptions.
     """
+
     def __init__(self, msg, do_log=True):
         if do_log:
             gc3libs.log.critical(msg)
@@ -107,6 +118,7 @@ class FatalError(UnrecoverableError):
 ## derived exceptions
 
 class AuthError(Error):
+
     """
     Base class for Auth-related errors.
 
@@ -119,11 +131,13 @@ class AuthError(Error):
 class RecoverableAuthError(AuthError, RecoverableError):
     pass
 
+
 class UnrecoverableAuthError(AuthError, UnrecoverableError):
     pass
 
 
 class ConfigurationError(FatalError):
+
     """
     Raised when the configuration file (or parts of it) could not be
     read/parsed.  Also used to signal that a required parameter is
@@ -133,6 +147,7 @@ class ConfigurationError(FatalError):
 
 
 class DataStagingError(Error):
+
     """
     Base class for data staging and movement errors.
 
@@ -141,7 +156,9 @@ class DataStagingError(Error):
     """
     pass
 
+
 class RecoverableDataStagingError(DataStagingError, RecoverableError):
+
     """
     Raised when transient problems with copying data to or from the
     remote execution site occurred.
@@ -152,15 +169,17 @@ class RecoverableDataStagingError(DataStagingError, RecoverableError):
     """
     pass
 
+
 class UnrecoverableDataStagingError(DataStagingError, UnrecoverableError):
+
     """
     Raised when problems with copying data to or from the remote
     execution site occurred.
     """
     pass
 
-
 class InputFileError(FatalError):
+
     """
     Raised when an input file is specified, which does not exist or
     cannot be read.
@@ -168,7 +187,8 @@ class InputFileError(FatalError):
     pass
 
 
-class InternalError(Error):
+class InternalError(Error, AssertionError):
+
     """
     Raised when some function cannot fulfill its duties, for reasons
     that do not depend on the library client code.  For instance, when
@@ -177,7 +197,9 @@ class InternalError(Error):
     """
     pass
 
+
 class AuxiliaryCommandError(InternalError):
+
     """
     Raised when some external command that we depend upon has failed.
 
@@ -187,7 +209,8 @@ class AuxiliaryCommandError(InternalError):
     pass
 
 
-class InvalidArgument(Error, AssertionError): # XXX: should this be fatal?
+class InvalidArgument(Error, AssertionError):  # XXX: should this be fatal?
+
     """
     Raised when the arguments passed to a function do not honor some
     required contract.  For instance, either one of two optional
@@ -195,21 +218,27 @@ class InvalidArgument(Error, AssertionError): # XXX: should this be fatal?
     """
     pass
 
+
 class InvalidType(InvalidArgument, TypeError):
+
     """
     A specialization of`InvalidArgument` for cases when the type of
     the passed argument does not match expectations.
     """
     pass
 
+
 class InvalidValue(InvalidArgument, ValueError):
+
     """
-    A specialization of`InvalidArgument` for cases when the type of
+    A specialization of`InvalidArgument` for cases when the value of
     the passed argument does not match expectations.
     """
     pass
 
+
 class DuplicateEntryError(InvalidArgument):
+
     """
     Raised by `Application.__init__` if not all (local or remote)
     entries in the input or output files are distinct.
@@ -218,6 +247,7 @@ class DuplicateEntryError(InvalidArgument):
 
 
 class InvalidOperation(Error):
+
     """
     Raised when an operation is attempted, that is not considered
     valid according to the system state.  For instance, trying to
@@ -227,14 +257,21 @@ class InvalidOperation(Error):
 
 
 class InvalidResourceName(Error, ValueError):
+
     """
     Raised to signal that no computational resource with the given
     name is defined in the configuration file.
+
+    Raising this exception will automatically log its message at ERROR
+    level, unless the `do_log=False` optional argument is explicitly
+    passed to the constructor.
     """
-    pass
+    def __init__(self, msg, do_log=True):
+        super(InvalidResourceName, self).__init__(msg, do_log)
 
 
 class InvalidUsage(FatalError):
+
     """
     Raised when a command is not provided all required arguments on
     the command line, or the arguments do not match the expected
@@ -247,6 +284,7 @@ class InvalidUsage(FatalError):
 
 
 class LoadError(Error):
+
     """
     Raised upon errors loading a job from the persistent storage.
     """
@@ -261,23 +299,51 @@ class LRMSSubmitError(LRMSError):
     pass
 
 
-class LRMSSkipSubmissionToNextIteration(LRMSSubmitError, RecoverableError):
+class ResourceNotReady(LRMSSubmitError, RecoverableError):
+
     """
-    An elastic resource has initiated adapting for a new task.
-    Although we cannot submit the task right now, it *will* be
-    accepted in the (not too distant) future.
+    A resource is not yet ready to accept tasks.
+
+    For instance: a new virtual machine has been started to run for a
+    task, but it is still booting.  Although we cannot submit the task
+    right now, it *will* be accepted in the (not too distant) future.
     """
     pass
 
 
+class LRMSSkipSubmissionToNextIteration(ResourceNotReady):
+
+    """
+    Older and deprecated alias for `ResourceNotReady`:class:
+
+    Only actually kept for backwards-compatibility.
+    """
+    def __init__(self, msg, do_log=False):
+        warn(
+            "Old class name `LRMSSkipSubmissionToNextIteration` called."
+            " Please fix the source code to use `ResourceNotReady` instead.",
+            DeprecationWarning, 2)
+        super(LRMSSkipSubmissionToNextIteration, self).__init__(msg, do_log)
+
+
 class MaximumCapacityReached(LRMSSubmitError, RecoverableError):
+
     """
     Indicates that a resource is full and cannot run any more jobs.
     """
     pass
 
 
-class NoConfigurationFile(FatalError):
+class ConfigurationFileError(FatalError):
+
+    """
+    Generic issue with the configuration file(s).
+    """
+    pass
+
+
+class NoConfigurationFile(ConfigurationFileError):
+
     """
     Raised when the configuration file cannot be read (e.g., does not
     exist or has wrong permissions), or cannot be parsed (e.g., is
@@ -286,7 +352,25 @@ class NoConfigurationFile(FatalError):
     pass
 
 
+class NoAccessibleConfigurationFile(NoConfigurationFile):
+
+    """
+    Raised when the configuration file cannot be read (e.g., does not
+    exist or has wrong permissions).
+    """
+    pass
+
+
+class NoValidConfigurationFile(NoConfigurationFile):
+    """
+    Raised when the configuration file cannot be parsed (e.g., is
+    malformed).
+    """
+    pass
+
+
 class NoResources(Error):
+
     """
     Raised to signal that no resources are defined, or that none are
     compatible with the request.
@@ -296,6 +380,7 @@ class NoResources(Error):
 
 
 class OutputNotAvailableError(InvalidOperation):
+
     """
     Raised upon attempts to retrieve the output for jobs that are
     still in `NEW` or `SUBMITTED` state.
@@ -303,14 +388,25 @@ class OutputNotAvailableError(InvalidOperation):
     pass
 
 
+class SpoolDirError(LRMSError, InvalidValue, UnrecoverableError):
+
+    """
+    Raised when a backend fails to access the spooldir either because
+    it does not exists or cannot be read.
+    """
+    pass
+
+
 class TaskError(Error):
+
     """
     Generic error condition in a `Task` object.
     """
     pass
 
 
-class DetachedFromGridError(TaskError):
+class DetachedFromControllerError(TaskError):
+
     """
     Raised when a method (other than :meth:`attach`) is called on
     a detached `Task` instance.
@@ -319,6 +415,7 @@ class DetachedFromGridError(TaskError):
 
 
 class UnexpectedStateError(TaskError):
+
     """
     Raised by :meth:`Task.progress` when a job lands in `STOPPED`
     or `TERMINATED` state.
@@ -326,19 +423,24 @@ class UnexpectedStateError(TaskError):
     pass
 
 
-class TransportError(Error):
+class TransportError(Error, EnvironmentError):
     pass
 
+
 class RecoverableTransportError(RecoverableError):
     pass
 
+
 class UnrecoverableTransportError(UnrecoverableError):
     pass
 
+
 class CopyError(TransportError):
+
     """
-    Error copying a file from `source` to `destination.
+    Error copying a file from `source` to `destination`.
     """
+
     def __init__(self, source, destination, ex):
         self.source = source
         self.destination = destination
@@ -347,7 +449,9 @@ class CopyError(TransportError):
             "Could not copy '%s' to '%s': %s: %s"
             % (source, destination, ex.__class__.__name__, str(ex)))
 
+
 class UnknownJob(Error, ValueError):
+
     """
     Raised when an operation is attempted on a task, which is
     unknown to the remote server or backend.
@@ -355,7 +459,24 @@ class UnknownJob(Error, ValueError):
     pass
 
 
+class UnexpectedJobState(RecoverableError, ValueError):
+    """
+    Raised when a job state is gotten from the execution code, that
+    does not match what GC3Pie expects for the task.
+
+    Typically this is a synchronization issue (different parts of a
+    system update at different times), hence this error is marked as
+    "recoverable".
+
+    For instance, a task might be ``TERMINATED`` according to GC3Pie
+    but the batch system accounting commands still report it as
+    running.
+    """
+    pass
+
+
 class UnknownJobState(Error, AssertionError):
+
     """
     Raised when a job state is gotten from the Grid middleware, that
     is not handled by the GC3Libs code.  Might actually mean that
@@ -364,15 +485,18 @@ class UnknownJobState(Error, AssertionError):
     """
     pass
 
+
 class ApplicationDescriptionError(FatalError):
+
     """
-    Raised when the dumped description on a given Application produces something that the LRMS backend cannot process.
-    As an example, for arc backends, this error is raised when the parsing of the Application's XRSL fails
+    Raised when the dumped description on a given Application produces
+    something that the LRMS backend cannot process.
+
     """
     pass
 
 
-## main: run tests
+# main: run tests
 
 if "__main__" == __name__:
     import doctest
