@@ -2,7 +2,7 @@
 #
 """
 """
-# Copyright (C) 2012, GC3, University of Zurich. All rights reserved.
+# Copyright (C) 2012  University of Zurich. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -15,17 +15,17 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
+from __future__ import absolute_import, print_function, unicode_literals
 __docformat__ = 'reStructuredText'
-__version__ = '$Revision$'
 
 from gc3libs.cmdline import SessionBasedScript
 from gc3libs import Application
 
 
 class SimpleScript(SessionBasedScript):
+
     """stupid class"""
     version = '1'
 
@@ -35,7 +35,7 @@ class SimpleScript(SessionBasedScript):
         return [
             # old-style
             ('MyJob',
-               Application,
+             Application,
                [
                    # arguments
                    ('/bin/bash', '-c', 'echo ciao > SimpleScript.stdout'),
@@ -43,7 +43,7 @@ class SimpleScript(SessionBasedScript):
                    [],
                    # outputs
                    ['SimpleScript.stdout'],
-                 ],
+               ],
              dict(output_dir='SimpleScript.out.d', join=True)),
             # new style, with explicit output dir
             Application(
@@ -58,7 +58,7 @@ class SimpleScript(SessionBasedScript):
                 # extra args
                 join=True,
                 **extra
-                ),
+            ),
             # new style, with output dir gotten from `self.extra`
             Application(
                 # arguments
@@ -68,12 +68,12 @@ class SimpleScript(SessionBasedScript):
                 # outputs
                 ['SimpleScript.stdout'],
                 join=True,
-                output_dir = default_output_dir,
+                output_dir=default_output_dir,
                 **extra
-                ),
-            ]
+            ),
+        ]
 
-## main: run tests
+# main: run tests
 
 if "__main__" == __name__:
     app = SimpleScript()
